Package: SigCheck
Type: Package
Title: Check a gene signature's prognostic performance against random
        signatures, known signatures, and permuted data/metadata
Version: 2.44.0
Author: Rory Stark <bioconductor@starkhome.com> and 
         Justin Norden 
Maintainer: Rory Stark <bioconductor@starkhome.com>
Description: While gene signatures are frequently used to predict phenotypes
        (e.g. predict prognosis of cancer patients), it it not always
        clear how optimal or meaningful they are (cf David Venet,
        Jacques E. Dumont, and Vincent Detours' paper "Most Random Gene
        Expression Signatures Are Significantly Associated with Breast
        Cancer Outcome"). Based on suggestions in that paper,
        SigCheck accepts a data set (as an ExpressionSet) and a gene
        signature, and compares its performance on survival and/or 
        classification tasks against 
        a) random gene signatures of the same length; 
        b) known, related and unrelated gene signatures; 
        and c) permuted data and/or metadata.
License: Artistic-2.0
LazyLoad: yes
Depends: R (>= 4.0.0), MLInterfaces, Biobase, e1071, BiocParallel,
        survival
Imports: graphics, stats, utils, methods
Suggests: BiocStyle, breastCancerNKI, qusage
biocViews: GeneExpression, Classification, GeneSetEnrichment
git_url: https://git.bioconductor.org/packages/SigCheck
git_branch: RELEASE_3_23
git_last_commit: 3d8182f
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 00:42:17 UTC; biocbuild

#import(MLInterfaces, Biobase, e1071, BiocParallel)

export(sigCheck)
export(sigCheckRandom)
export(sigCheckPermuted)
export(sigCheckKnown)
export(sigCheckPlot)
export(sigCheckAll)

#exportMethods(
# "checkType",                          
# "classes",                            
# "annotation",                         
# "survival",                           
# "signature",                          
# "validationSamples",                  
# "survivalMethod",                     
# "threshold",                          
# "survivalType",                       
# "timeUnits",                          
# "survivalScores",                     
# "survivalConfusionMatrix",            
# "survivalClassificationScore",        
# "survivalPval",                       
# "survivalTrainingScores",             
# "survivalTrainingConfusionMatrix",    
# "survivalTrainingClassificationScore",
# "survivalTrainingPval",               
# "classifierMethod",                   
# "sigPerformance",                     
# "confusion",            
# "modeVal",
# "modePerformance",                    
# "classifier"  
#)

exportClasses(
    "SigCheckObject"
)


#Imports: AnnotationDbi, Biobase, BiocGenerics, BiocParallel, graphics,
#         IRanges, MLInterfaces, S4Vectors, stats, utils

importMethodsFrom(Biobase, exprs, "exprs<-", featureData, varLabels,
                  varMetadata, sampleNames)

importMethodsFrom(BiocParallel, bplapply)

importMethodsFrom(MLInterfaces, confuMat, MLearn)

importFrom(graphics, abline, legend, par)

importFrom(MLInterfaces, xvalSpec)

importFrom(stats, ecdf, formula)

importFrom(utils, data)

importFrom(MLInterfaces, svmI, trainScores, testScores)

importFrom(survival, Surv, survfit, survdiff)

importFrom(e1071, svm)

import(methods)

importFrom("graphics", "plot", "text")

importFrom("stats", "density", "median", "pchisq", "prcomp",
             "quantile")

setClass("SigCheckObject",contains = "ExpressionSet",
         slots=c(
             checkType="character",
             classes="character",
             annotation="character",
             survival="character",
             signature="vector",
             signatureLabels="vector",
             validationSamples="vector",  
             survivalMethod="ANY",
             threshold="ANY", 
             survivalScores="numeric",  
             survivalConfusionMatrix="matrix",
             survivalClassificationScore="numeric",  
             survivalPval="numeric",  
             survivalTrainingScores="numeric",  
             survivalTrainingConfusionMatrix="matrix",
             survivalTrainingClassificationScore="numeric",  
             survivalTrainingPval="numeric",
             survivalLabel="character", 
             timeLabel="character",
             classifierMethod="learnerSchema",
             sigPerformance="numeric", 
             confusion="ANY",
             modeVal="character", 
             modePerformance="numeric", 
             classifier="classifierOutput"
         ))


setMethod("show","SigCheckObject",
          function (object){
              message(class(object)," ")
              message("Type: ",object@checkType," ")
              if(object@checkType=="Classifier") {
                  message("Classifier performance: ",
                          sprintf("%1.2f",object@sigPerformance),
                          ". Confusion matrix: ")
                  print(object@confusion)
              }
              if(length(object@survivalPval)) {
                  message("Survival p-value: ",
                          sprintf("%1.4e",object@survivalPval)," ")
              }
              print(as(object,"ExpressionSet"))
          }
)

setGeneric('checkType', function(object='SigCheckObject') 
    standardGeneric('checkType'))
setMethod('checkType', 'SigCheckObject', 
          function(object) object@checkType)
setGeneric('classes', function(object='SigCheckObject') 
    standardGeneric('classes'))
setMethod('classes', 'SigCheckObject', 
          function(object) object@classes)
setGeneric('annotation', function(object='SigCheckObject') 
    standardGeneric('annotation'))
setMethod('annotation', 'SigCheckObject', 
          function(object) object@annotation)
setGeneric('survival', function(object='SigCheckObject') 
    standardGeneric('survival'))
setMethod('survival', 'SigCheckObject', 
          function(object) object@survival)
setGeneric('signature', function(object='SigCheckObject') 
    standardGeneric('signature'))
setMethod('signature', 'SigCheckObject',
          function(object) object@signature)
setGeneric('validationSamples', function(object='SigCheckObject')
    standardGeneric('validationSamples'))
setMethod('validationSamples', 'SigCheckObject', 
          function(object) object@validationSamples)
setGeneric('survivalMethod', function(object='SigCheckObject') 
    standardGeneric('survivalMethod'))
setMethod('survivalMethod', 'SigCheckObject', 
          function(object) object@survivalMethod)
setGeneric('threshold', function(object='SigCheckObject') 
    standardGeneric('threshold'))
setMethod('threshold', 'SigCheckObject',
          function(object) object@threshold)
setGeneric('survivalType', function(object='SigCheckObject') 
    standardGeneric('survivalType'))
setMethod('survivalType', 'SigCheckObject',
          function(object) object@survivalType)
setGeneric('timeUnits', function(object='SigCheckObject') 
    standardGeneric('timeUnits'))
setMethod('timeUnits', 'SigCheckObject', 
          function(object) object@timeUnits)
setGeneric('survivalScores', function(object='SigCheckObject') 
    standardGeneric('survivalScores'))
setMethod('survivalScores', 'SigCheckObject', 
          function(object) object@survivalScores)
setGeneric('survivalConfusionMatrix', function(object='SigCheckObject')
    standardGeneric('survivalConfusionMatrix'))
setMethod('survivalConfusionMatrix', 'SigCheckObject',
          function(object) object@survivalConfusionMatrix)
setGeneric('survivalClassificationScore', function(object='SigCheckObject')
    standardGeneric('survivalClassificationScore'))
setMethod('survivalClassificationScore', 'SigCheckObject', 
          function(object) object@survivalClassificationScore)
setGeneric('survivalPval', function(object='SigCheckObject') 
    standardGeneric('survivalPval'))
setMethod('survivalPval', 'SigCheckObject', 
          function(object) object@survivalPval)
setGeneric('survivalTrainingScores', function(object='SigCheckObject') 
    standardGeneric('survivalTrainingScores'))
setMethod('survivalTrainingScores', 'SigCheckObject',
          function(object) object@survivalTrainingScores)
setGeneric('survivalTrainingConfusionMatrix', function(object='SigCheckObject')
    standardGeneric('survivalTrainingConfusionMatrix'))
setMethod('survivalTrainingConfusionMatrix', 'SigCheckObject', 
          function(object) object@survivalTrainingConfusionMatrix)
setGeneric('survivalTrainingClassificationScore', 
           function(object='SigCheckObject') 
               standardGeneric('survivalTrainingClassificationScore'))
setMethod('survivalTrainingClassificationScore', 'SigCheckObject',
          function(object) object@survivalTrainingClassificationScore)
setGeneric('survivalTrainingPval', function(object='SigCheckObject')
    standardGeneric('survivalTrainingPval'))
setMethod('survivalTrainingPval', 'SigCheckObject', 
          function(object) object@survivalTrainingPval)
setGeneric('classifierMethod', function(object='SigCheckObject') 
    standardGeneric('classifierMethod'))
setMethod('classifierMethod', 'SigCheckObject', 
          function(object) object@classifierMethod)
setGeneric('sigPerformance', function(object='SigCheckObject')
    standardGeneric('sigPerformance'))
setMethod('sigPerformance', 'SigCheckObject',
          function(object) object@sigPerformance)
setGeneric('confusion', function(object='SigCheckObject') 
    standardGeneric('confusion'))
setMethod('confusion', 'SigCheckObject',
          function(object) object@confusion)
setGeneric('modeVal', function(object='SigCheckObject')
    standardGeneric('modeVal'))
setMethod('modeVal', 'SigCheckObject', 
          function(object) object@modeVal)
setGeneric('modePerformance', function(object='SigCheckObject') 
    standardGeneric('modePerformance'))
setMethod('modePerformance', 'SigCheckObject',
          function(object) object@modePerformance)
setGeneric('classifier', function(object='SigCheckObject') 
    standardGeneric('classifier'))
setMethod('classifier', 'SigCheckObject', 
          function(object) object@classifier)

# Plot check checkResults
sigCheckPlot = function(checkResults, classifier=FALSE, 
                        title, nolegend=FALSE, ...) {
   if( (length(checkResults)== 4) &&
       (all(unlist(lapply(checkResults,function(x)x$checkType)) == 
            c("Random","Known",
              "Permuted", "Permuted")))) {
      savemfrow = par("mfrow")
      par(mfrow=c(2,2))
      for(tocheck in checkResults) {
         sigCheckPlot(tocheck,classifier,title,nolegend, ...)
      }
      par(mfrow=savemfrow)
   } else {   
      if(classifier) {
         if(is.null(checkResults$sigPerformance)) {
            stop("No classification results present!")
         } else {
            sigCheckPlotClassifier(checkResults,title,nolegend, ...)
         }
      } else {
         sigCheckPlotSurvival(checkResults,title,nolegend, ...)
      }
   }
}

sigCheckPlotClassifier <- function(checkResults,title,nolegend=FALSE, ...){
   
   if( (length(checkResults) == 4) &&
       (names(checkResults) == c("checkClassifier","checkRandom",
                                 "checkKnown","checkPermutedFeatures",
                                 "checkPermutedCategories")) ) {
      for(i in 2:5) {
         sigCheckPlot(checkResults[[i]])
      }
   } else {
      scores <- sort(checkResults$performance)
      fields <- names(checkResults)
      
      if(missing(title)) {
         if ("performanceRandom" %in% fields) {
            titlestring <- sprintf("Check: Random Signatures")
         }
         if ("performanceKnown" %in% fields) {
            titlestring <- sprintf("Check: Known Signatures [%s]",
                                   checkResults$known)
         }
         if ("performancePermuted" %in% fields) {
            titlestring <- sprintf("Check: Permuted Data: [%s]",
                                   checkResults$permute)
         }
      } else {
         titlestring <- title
      }
      xmin <- min(scores,checkResults$sigPerformance,
                  checkResults$modePerformance)
      xmax <- max(scores,checkResults$sigPerformance,
                  checkResults$modePerformance)    
      pval <- 1 - (sum(checkResults$sigPerformance > scores)/length(scores))
      accuracy <- ceiling(log10(length(scores)))
      if(pval > 0) {
         rankstr <- sprintf("Percentile:%%.2f (Tests:%%d  p=%%1.%df)",
                            accuracy)
      } else {
         rankstr <- sprintf("Percentile:%%.2f (Tests:%%d  p<%%1.%df)",
                            accuracy)
         pval <- 1/(10^accuracy)
      }
      rankstr <- sprintf(rankstr,ecdf(scores)(checkResults$sigPerformance),
                         length(scores),pval)
      
      plot(density(scores),
           main=titlestring,sub=rankstr,...)
      abline(v=checkResults$sigPerformance,col="red",lwd=3)
      abline(v=checkResults$modePerformance,col="red",lty="dotted",lwd=2)
      if(!nolegend){
         legend("topright",legend=c(sprintf("Signature [%0.3f]",
                                            checkResults$sigPerformance),
                                    sprintf("Mode [%0.3f]",
                                            checkResults$modePerformance)),
                col="red",lty=c("solid","dotted"),lwd=c(3,2))
      }
      
   }
}

sigCheckPlotSurvival <- function(checkResults,title,nolegend=FALSE,...){
   
   if( (length(checkResults)== 5) &&
       (names(checkResults) == c("checkClassifier","checkRandom",
                                 "checkKnown","checkPermutedFeatures",
                                 "checkPermutedCategories")) ) {
      for(i in 2:5) {
         sigCheckPlotSurvival(checkResults[[i]])
      }
   } else {
      scores <- checkResults$survivalPvals
      if(is.null(scores)) {
         stop("No survival scores computed.")
      }
      scores <- sort(scores)
      fields <- names(checkResults)
      
      if(missing(title)) {
         if (checkResults$checkType == "Random") {
            titlestring <- sprintf("Survival: Random Signatures")
         }
         if (checkResults$checkType == "Known") {
            titlestring <- sprintf("Survival: Known Signatures [%s]",
                                   checkResults$known)
         }
         if (checkResults$checkType == "Permuted") {
            titlestring <- sprintf("Survival: Permuted Data: [%s]",
                                   checkResults$permute)
         } 
      } else {
         titlestring <- title
      }
      xmin <- min(scores)
      xmax <- max(scores)    
      pval <- 1 - (sum(checkResults$survivalPval < scores)/length(scores))
      accuracy <- ceiling(log10(length(scores)))
      
      if(pval > 0) {
         rankstr <- sprintf("Percentile:%%.2f (Tests:%%d  p=%%1.%df)",
                            accuracy)
         pvalstr <- sprintf("Significant p-val =%%1.%df",accuracy)
      } else {
         rankstr <- sprintf("Percentile:%%.2f (Tests:%%d  p<%%1.%df)",
                            accuracy)
         pvalstr <- sprintf("Signature   p-val <%%1.%df",accuracy)        
         pval <- 1/(10^accuracy)
      }
      
      if(checkResults$survivalPval > 0.001) {
         sigpvalstr <- "Signature   p-val =%1.3f (%1.2f)"
         pvalSurvival <- checkResults$survivalPval
      } else {
         sigpvalstr <- "Signature   p-val <%1.3f (%1.2f)"        
         pvalSurvival <- 1/(10^3)
      }
      
      rankstr <- sprintf(rankstr,1-ecdf(scores)(checkResults$survivalPval),
                         length(scores),pval)
      
      plot(density(-log10(scores)),
           #xlim=c(log10(xmin),log10(xmax)),
           main=titlestring,sub=rankstr,...)
      #if(checkResults$survivalPval>xmin) {
      abline(v=-log10(checkResults$survivalPval),col="red",lwd=3)
      #}
      
      #if(0.05>xmin) {
      abline(v=-log10(.05),col="red",lty="dotted",lwd=2)
      #} 
      if(!nolegend) {
         legend("topright",
                legend=c(sprintf(sigpvalstr,pvalSurvival, 
                                 -log10(checkResults$survivalPval)),
                         "Significant p-val <0.05    (-1.30)"),
                col="red",lty=c("solid","dotted"),lwd=c(3,2))
      }
   }
}   
.sigCheckPval = function(performance,scores) {
   pval <- 1 - (sum(performance < scores)/length(scores))
   accuracy <- ceiling(log10(length(scores)))
   pval = signif(pval,accuracy)
   return(pval)
}

# Test Survival Profile  of Gene Signature
sigCheckSurvival <- function(SigCheck,
                             plotTrainingKM=TRUE, plotValidationKM=TRUE, 
                             timeLabel, survivalLabel,impute=FALSE, ...){
   
   category <- which(varLabels(SigCheck) %in% SigCheck@classes)
   if(length(unique(SigCheck[[category]])) != 2) {
      stop('ERROR: classes can specify only two categories for classification.')
   }
   if(class(SigCheck[[category]])!="factor") {
      SigCheck[[category]] <- as.factor(SigCheck[[category]])
   }
   classVals    <- .sigCheckPheno(SigCheck,SigCheck@classes,bAsFactor=TRUE)
   survivalVals <- .sigCheckPheno(SigCheck,SigCheck@survival,bAsFactor=FALSE)
   
   
   
   trainingSet <- 1:length(colnames(SigCheck))
   if(length(SigCheck@validationSamples)){
      trainingSet <- trainingSet[-SigCheck@validationSamples]
   } 
   
   SigCheck <- .sigCheckNA(SigCheck,impute=impute)
   
   #Subset Whole Matrix Based Off of GeneSigIndices
   saveSig <- SigCheck@signature
   SigCheck@signature <- .sigCheckSignature(SigCheck, SigCheck@signature, 
                                            SigCheck@annotation)
   
   
   survivalScores=.sigCheckSurvival(SigCheck, SigCheck@signature,
                                    SigCheck@survival, SigCheck@survivalMethod,
                                    trainingSet,SigCheck@validationSamples)
   
   ## confusion matrices: use classes to compute matrices for training/validation
   trainingPredictions <- 
      .sigCheckSurvivalClassify(survivalScores$trainingScores,
                                classVals[trainingSet],SigCheck@threshold)
   trainingConfusionMatrix <- 
      .sigCheckSurvivalConfuse(trainingPredictions,
                               classVals[trainingSet])
   
   if(length(SigCheck@validationSamples)) {
      validationPredictions <- 
         .sigCheckSurvivalClassify(survivalScores$validationScores,
                                   classVals[SigCheck@validationSamples],
                                   SigCheck@threshold)
      validationConfusionMatrix <-
         .sigCheckSurvivalConfuse(validationPredictions,
                                  classVals[SigCheck@validationSamples])
   }
   
   ## compute scores from matrix for training/validation
   trainingScore <- 
      .sigCheckSurvivalClassificationScore(trainingConfusionMatrix)
   if(length(SigCheck@validationSamples)) {
      validationScore <-  
         .sigCheckSurvivalClassificationScore(validationConfusionMatrix)
   }
   
   ## KM plots
   
   savemfrow=NULL
   if(length(SigCheck@validationSamples)) {
      if(plotTrainingKM & plotValidationKM) {
         savemfrow <- par("mfrow")
         par(mfrow=c(1,2))
      }
   } 
   if(length(SigCheck@validationSamples)) {
      mainstr <- "Survival: Training Set"
   } else {
      mainstr <- "Survival: Full Set"
   }
   categories <- 
      .sigCheckSurvivalCategories(survivalScores$trainingScores,
                                  SigCheck@threshold)    
   if(length(unique(categories))>1){
      trainingPval <- .sigCheckPlotKM(survivalVals[trainingSet],
                                      classVals[trainingSet],
                                      categories,
                                      pvalOnly=!plotTrainingKM, 
                                      xlab=SigCheck@timeLabel, 
                                      ylab=SigCheck@survivalLabel,
                                      main=mainstr)
      
   } else {
      if(plotTrainingKM) {
         warning("No KM plot for training samples: all samples in same category.")
      }
      trainingPval=1
   }
   if(length(SigCheck@validationSamples)){
      categories <- 
         .sigCheckSurvivalCategories(survivalScores$validationScores,
                                     SigCheck@threshold)
      if(length(unique(categories))>1){
         validationPval <- .sigCheckPlotKM(survivalVals[SigCheck@validationSamples],
                                           classVals[SigCheck@validationSamples],
                                           categories,
                                           pvalOnly=!plotValidationKM, 
                                           xlab=SigCheck@timeLabel, 
                                           ylab=SigCheck@survivalLabel,
                                           main="Survival: Validation Set") 
      } else {
         if(plotValidationKM) {
            warning("No KM plot for validation samples: all samples in same category.")
         }
         validationPval=1
      }
   } else validationPval=1
   
   if(!is.null(savemfrow)) {
      par(mfrow=savemfrow)
   }
   
   ## perfect survival: survivors vs. non-survivors
   
   SigCheck@checkType <- "Survival"
   
   if(length(SigCheck@validationSamples)) {
      SigCheck@survivalScores              <- survivalScores$validationScores
      SigCheck@survivalConfusionMatrix     <- validationConfusionMatrix
      SigCheck@survivalClassificationScore <- validationScore
      SigCheck@survivalPval                <- validationPval
      SigCheck@survivalTrainingScores              <- survivalScores$trainingScores
      SigCheck@survivalTrainingConfusionMatrix     <- trainingConfusionMatrix
      SigCheck@survivalTrainingClassificationScore <- trainingScore
      SigCheck@survivalTrainingPval                <- trainingPval
   } else {
      SigCheck@survivalScores              <- survivalScores$trainingScores
      SigCheck@survivalConfusionMatrix     <- trainingConfusionMatrix
      SigCheck@survivalClassificationScore <- trainingScore
      SigCheck@survivalPval                <- trainingPval
   }
   
   return(SigCheck)
}

.sigCheckSurvival <- function(expressionSet, signature,
                              survival,survivalMethod,
                              trainingSet,validationSamples) {
   
   survivalValues <- which(varLabels(expressionSet) %in% survival)
   subsetExpressionSet <- expressionSet[signature,trainingSet]
   if(!is.character(survivalMethod)){
      trainingScores <- 
         survivalMethod(subsetExpressionSet)
      if(length(validationSamples)) {
         subsetExpressionSet <- expressionSet[signature,validationSamples]
         validationScores <- 
            survivalMethod(subsetExpressionSet)
      } else {
         validationScores <- trainingScores
      }  
   } else {
      if(survivalMethod=="PCA1") {
         trainingScores <- 
            .sigCheckPCA1(subsetExpressionSet)
         if(length(validationSamples)) {
            subsetExpressionSet <- expressionSet[signature,validationSamples]
            validationScores <- 
               .sigCheckPCA1(subsetExpressionSet)
         } else {
            validationScores <- trainingScores
         }
      } else if (survivalMethod=="High") {
         trainingScores <- 
            .sigCheckHigh(subsetExpressionSet)
         if(length(validationSamples)) {
            subsetExpressionSet <- expressionSet[signature,validationSamples]
            validationScores <- 
               .sigCheckHigh(subsetExpressionSet)
         } else {
            validationScores <- trainingScores
         }    
      } else {
         stop("Invalid scoreMethod: ",survivalMethod)  
      }
   }
   return(list(trainingScores=trainingScores,
               validationScores=validationScores))
}

.sigCheckPCA1 <- function(expressionSet){
   pcs <- prcomp(t(exprs(expressionSet)))
   pc1 <- pcs$x[,1]
   return(pc1)
}


.sigCheckHigh <- function(expressionSet,fn=mean){
   means <- apply(exprs(expressionSet),2,fn)
   return(means)
}

.sigCheckSurvivalClassify <- function(scores,classVals,threshold,bFlip=TRUE){
   categories <- .sigCheckSurvivalCategories(scores,threshold)
   predict <- scores == "High"
   if(bFlip) {
      actual <- classVals == levels(classVals)[2]
      correct <- predict == actual
      if(sum(correct) > sum(!correct)) {
         return(predict)
      } else {
         return(!predict)
      }
   } else {
      return(predict)
   }
}

.sigCheckSurvivalConfuse <- function(predictions,classVals) {
   vals <- unique(classVals)
   wrong <- rep(vals[1],length(classVals))
   wrong[classVals==vals[1]] <- vals[2]
   predicted <- classVals
   predicted[!predictions] <- wrong[!predictions]
   tn <- sum(classVals==vals[1] & classVals==predicted)
   fn <- sum(classVals==vals[2] & classVals!=predicted)
   tp <- sum(classVals==vals[2] & classVals==predicted)
   fp <- sum(classVals==vals[1] & classVals!=predicted)
   
   confu <- matrix(c(tn,fn,fp,tp),2,2)
   rownames(confu) <- 0:1
   colnames(confu) <- 0:1
   return(confu)
}

.sigCheckSurvivalClassificationScore <- function(confusion) {
   total <- sum(confusion)
   correct <- confusion[1,1] + confusion[2,2]
   score <- correct/total
   return(score)
}

.sigCheckPlotKM <- function(y, status, x,
                            barcol=c("blue", "red", "grey"),
                            pvalOnly=FALSE,twogroups=TRUE, ...) {
   status <- as.numeric(status)-1
   
   if(twogroups) {
      mid <- x == "Mid"
   } else {
      mid <- rep(FALSE,length(x))
   }
   xx = x[!mid]
   yy = y[!mid]
   ss = status[!mid]
   
   
   pval <- pchisq(survdiff(Surv(yy, ss) ~xx)$chisq, 
                  length(unique(xx[!is.na(xx)]))-1, lower.tail=FALSE)
   
   if(!pvalOnly) {
      
      plot(survfit(Surv(y, status) ~ x), col=barcol,...)
      
      if (pval < 0.001) {
         pvalStr <- "<0.001"
      } else {
         pvalStr <- paste("=", round(pval, 3))
      }
      text(x=max(y)/12, y=0.1, paste("p", pvalStr))
      p1 <- table(x[!is.na(status)])
      p2 <- tapply(status, x, sum, na.rm=TRUE)
      p2[is.na(p2)] <- 0
      text.leg <- paste(levels(x) , ": ", p1, "(", p2, ")", sep="")
      legend("topright", lty=rep(1,3), lwd=2, col=barcol, 
             legend=text.leg, bty="n")
   }
   return(pval)
}


.sigCheckSurvivalCategories <- function(scores,threshold) {
   if(is.function(threshold)) {
      threshold <- threshold(scores)    
   } else {
      threshold = sort(threshold)
      threshold <- as.numeric(quantile(scores,threshold))
   }
   two <- rep(TRUE,length(scores))
   if(length(threshold)==1) {
      predict <- scores > threshold
      categories <- rep("Low",length(predict))
      categories[predict] <- "High"
   } else if(length(threshold)==2){
      low  <- scores <  threshold[1]
      high <- scores >= threshold[2]
      mid  <- !(low | high)
      categories <- rep("Low",length(scores))
      categories[high] <- "High"
      categories[mid] <- "Mid"
      two[mid] <- FALSE
   } else {
      stop("At most three categories supported")
      categories=rep(length(threshold)+1,length(scores))
      for(i in length(threshold):1) {
         categories[scores<threshold[i]]=i   
      }
   }
   categories <- factor(categories)   
   return(categories)
}

# Test Classification Potential of Gene Signature against 
#  Expression Set Permutations
# 
# Compares how an input gene signature classifies an expression set object. 
# Compares this classification using an SVM LOO CV analysis to taking random 
# gene signatures of equal length 
# to the input list and testing classification power.

sigCheckPermuted <- function(check, toPermute="categories", iterations=10){
    bParallel <- TRUE
    
    
    expressionSet     <- check
    classes           <- check@classes
    signature         <- check@signature
    annotation        <- check@annotation 
    validationSamples <- check@validationSamples 
    survival          <- check@survival
    threshold         <- check@threshold
    nosecond          <- FALSE
    
    if(check@checkType=="Classifier") {
        classifier         <- check
        modeVal            <- check@modeVal
        Method             <- check@classifierMethod
        classifierScore    <- check@sigPerformance
        classifierSurvival <- check@survivalPval
        if(!length(validationSamples)) {
            bParallel <- FALSE
            nosecond  <- TRUE
            validationSamples <- xvalSpec("LOO")
        } 
        if(survival=="") {
            nosecond <- TRUE
        }
    } else if(check@checkType == "Survival") {
        Method           <- check@survivalMethod
        survivalLabel     <- check@survivalLabel
        timeLabel        <- check@timeLabel
        survivalPval     <- check@survivalPval
        if(!length(validationSamples)) {
            nosecond <- TRUE
        }
    } else {
        stop("Invalid SigCheck object.")
    }  
    
    #expressionSet <- .sigCheckNA(expressionSet)
    
    signature <- .sigCheckSignature(expressionSet, signature, annotation)
    
    #Permute Testing
    permuteResult1 <- c()
    permuteResult2 <- c()
    if(bParallel) {
        checkit <- 
            bplapply(1:iterations, .sigCheckPermuted, 
                     expressionSet, check@checkType, 
                     toPermute, classes, signature,
                     validationSamples, Method,
                     survival, threshold, nosecond)
        if(nosecond){
            permuteResult1  <- unlist(checkit)
        } else {
            permuteResult1 <- sapply(checkit,function(x){x$result1})
            permuteResult2 <- sapply(checkit,function(x){x$result2})
        }
    } else {
        permuteResult1  <- NULL
        permuteResult2 <- NULL
        for (i in 1:iterations){    
            checkit <- .sigCheckPermuted(i, expressionSet, check@checkType,
                                         toPermute, classes, signature,
                                         validationSamples, Method,
                                         survival, threshold, nosecond)        
            if(nosecond) {
                permuteResult1 <- c(permuteResult1, checkit)
            } else {
                permuteResult1 <- c(permuteResult1, checkit$result1)
                permuteResult2 <- c(permuteResult2, checkit$result2)
            }
        }
    }
    
    output <- NULL
    if(check@checkType=="Classifier") {
        allScoreValues <- c(classifierScore, permuteResult1)
        sortScores     <- sort(allScoreValues, decreasing=TRUE)
        geneListRank   <- mean(which(sortScores == classifierScore))
        checkPval      <- .sigCheckPval(classifierScore,permuteResult1,lt=FALSE)
        nullPerf <- 
            .sigCheckClassifierNull(expressionSet, classes, check@validationSamples,
                                    modeVal=modeVal)
        output <- list(checkType="Permuted",
                       sigPerformance=classifierScore, 
                       modePerformance=nullPerf,
                       checkPval=checkPval,
                       performancePermuted=permuteResult1)
    } else {
        allScoreValues <- c(survivalPval,permuteResult1)
        sortScores     <- sort(allScoreValues, decreasing=FALSE)
        geneListRank   <- mean(which(sortScores == survivalPval))
        checkPval      <- .sigCheckPval(survivalPval,permuteResult1,lt=TRUE)
        output$checkType             <- "Permuted"
        output$survivalPvalsPermuted <- permuteResult1
        output$survivalPval          <- survivalPval
        output$checkPval             <- checkPval
    }
    
    output$permute <- toPermute
    output$tests   <- iterations
    output$rank   <- geneListRank
    
    if(check@checkType=="Classifier") {
        if(!nosecond) {
            output$survivalPval <- classifierSurvival
            output$survivalPvalsPermuted <- permuteResult2
        }
    } else {
        if(!nosecond) {
            output$trainingPvalsPermuted <- permuteResult2
        }
    }
    
    return(output)
    
}

.sigCheckPermuted <- function(iter, permuteEset, checkType,
                              toPermute, classes, signature,
                              validationSamples, Method,
                              survival=NULL, threshold, nosecond) {    
    
    exMatrix <- exprs(permuteEset)
    #permuteEset <- expressionSet
    
    if ("features" %in% toPermute){
        mix <- t(apply(exMatrix, 1, sample))
        colnames(mix) <- colnames(exMatrix)
        exprs(permuteEset) <- mix
    }
    
    if ("samples" %in% toPermute){
        mix <- apply(exMatrix, 2, sample)
        rownames(mix) <- rownames(exMatrix)
        exprs(permuteEset) <- mix
    }
    
    if ("categories" %in% toPermute){
        category <- which(varLabels(permuteEset) %in% classes)
        if(is.null(validationSamples)) {
            permuteEset[[category]] <- sample(permuteEset[[category]])
        } else {
            permuteEset[[category]][validationSamples] <- 
                sample(permuteEset[[category]][validationSamples])
            permuteEset[[category]][-validationSamples] <- 
                sample(permuteEset[[category]][-validationSamples])
        }
    }
    
    if ("survival" %in% toPermute){
        if(is.null(survival)) {
            stop("No survival times specified.")
        }
        category <- which(varLabels(permuteEset) %in% classes)
        if(is.null(validationSamples)) {
            permuteEset[[category]] <- sample(permuteEset[[category]])
        } else {
            permuteEset[[category]][validationSamples] <- 
                sample(permuteEset[[category]][validationSamples])
            permuteEset[[category]][-validationSamples] <- 
                sample(permuteEset[[category]][-validationSamples])
        }
    }
    permuteEset@signature <- signature
    if(checkType=="Classifier") {
        classifier <- 
            sigCheckClassifier(permuteEset, 
                               plotTrainingKM=FALSE, plotValidationKM=FALSE)
        gc()
        if(nosecond) {
            return(result1=classifier@sigPerformance)        
        } else {
            return(list(result1=classifier@sigPerformance,
                        result2=classifier@survivalPval)) 
        }
    } else {
        check <- 
            suppressWarnings(
                sigCheckSurvival(permuteEset,
                                 plotTrainingKM=FALSE, plotValidationKM=FALSE)
            )
        gc()
        if(nosecond) {
            return(result1=check@survivalPval)        
        } else {
            return(list(result1=check@survivalPval,
                        result2=check@survivalTrainingPval)) 
        }   
    }    
}

sigCheck <- function(expressionSet, classes, survival, signature, 
                     annotation, validationSamples, 
                     scoreMethod="PCA1", threshold=median,
                     classifierMethod=svmI, modeVal,
                     survivalLabel="Survival", timeLabel="Time",
                     plotTrainingKM=TRUE, plotValidationKM=TRUE,
                     impute=TRUE){
   
   if(class(expressionSet)=="SigCheckObject") {
      if(missing(classes)) {
         classes <- expressionSet@classes
      }
      if(missing(survival)) {
         survival <- expressionSet@survival
      }
      if(missing(signature)) {
         signature <- expressionSet@signatureLabels
      }
      if(missing(annotation)) {
         annotation <- expressionSet@annotation
      }
      if(missing(validationSamples)) {
         validationSamples <- expressionSet@validationSamples
      }
      if(missing(scoreMethod)) {
         scoreMethod <- expressionSet@survivalMethod
      }
      if(missing(threshold)) {
         threshold <- expressionSet@threshold
      }
      if(missing(classifierMethod)) {
         classifierMethod <- expressionSet@classifierMethod
      }
      if(missing(modeVal)) {
         modeVal <- expressionSet@modeVal
      }
      if(missing(survivalLabel)) {
         survivalLabel <- expressionSet@survivalLabel
      }
      if(missing(timeLabel)) {
         timeLabel <- expressionSet@timeLabel
      }
      check <- sigCheck(as(expressionSet,"ExpressionSet"), 
                        classes, survival, signature, 
                        annotation, validationSamples, 
                        scoreMethod, threshold,
                        classifierMethod, modeVal,
                        survivalLabel, timeLabel,
                        plotTrainingKM, plotValidationKM,
                        impute)
      return(check)
   }
   
   if(missing(validationSamples)) {
      validationSamples <- numeric(0)
   }
   if(missing(annotation)) {
      annotation <- ""
   }
   if(missing(survival)) {
      survival <- ""
   }
   if(missing(survivalLabel)) {
      survivalLabel <- survival
   }
   if(missing(timeLabel)) {
      timeLabel <- ""
   }
   if(missing(timeLabel)) {
      timeLabel <- "Time"
   }
   if(missing(modeVal)) {
      modeVal <- ""
   }
   
   result = as(expressionSet,"SigCheckObject")
   result@classes <- classes
   result@annotation <- annotation
   result@survival <- survival
   result@signature <- signature
   result@validationSamples <- validationSamples
   result@threshold <- threshold
   result@classifierMethod <- classifierMethod
   result@survivalLabel <- survivalLabel
   result@timeLabel <- timeLabel
   result@survivalMethod <- scoreMethod
   result@modeVal <- modeVal
   
   if(!is.character(scoreMethod)) {
      scoreMethod="user"
   }
   
   if(scoreMethod=="classifier") {
      if(missing(survival)) {
         result@survival <- ""
      } else {
         result@survival <- survival
      }
      
      result@checkType <- "Classifier"
      result <- sigCheckClassifier(result,
                                   plotTrainingKM=plotTrainingKM,
                                   plotValidationKM=plotValidationKM,
                                   impute=impute)
   } else if(scoreMethod!="classifier")  {
      if(survival=="") {
         stop("Must specify a survival label.")
      } else {
         result@survival  <- survival
         result@checkType <- "Survival"
      }
      result <- sigCheckSurvival(result,
                                 modeVal=modeVal,
                                 plotTrainingKM=plotTrainingKM,
                                 plotValidationKM=plotValidationKM,
                                 impute=impute)       
   } else {
      stop("Invalid scoreMethod: [",scoreMethod,"]")
   }
   
   result@signatureLabels <- signature
   
   return(result)
}


sigCheckAll <- function(check,
                        iterations=10, 
                        known="cancer",
                        plotResults=TRUE, ...){
   
   message('Check random signatures...')
   randomGeneOutput  <-
      sigCheckRandom(check,iterations=iterations)
   
   
   message('Check known signatures...')
   knownGenesOutput  <- 
      sigCheckKnown(check,known=known)
   
   
   if(check@checkType=="Survival" || check@survival!="") {
      nosurv <- FALSE
      toPermute <- "survival"
   } else {
      nosurv <- TRUE
      toPermute <- "categories"
   }
   toPermute = c(toPermute,"features")    
   
   message(sprintf("Check permuted: [%s]...",toPermute[1]))   
   permute1Output <- 
      sigCheckPermuted(check, toPermute=toPermute[1],
                       iterations=iterations)
   message(sprintf("Check permuted: [%s]...",toPermute[2]))   
   permute2Output <- 
      sigCheckPermuted(check, toPermute=toPermute[2],
                       iterations=iterations) 
   
   if(toPermute[1]=="survival") {
      output <- list(checkRandom=randomGeneOutput,
                     checkKnown=knownGenesOutput,
                     checkPermutedSurvival=permute1Output,
                     checkPermutedFeatures=permute2Output)
   } else {
      output <- list(checkRandom=randomGeneOutput,
                     checkKnown=knownGenesOutput,
                     checkPermutedCategories=permute1Output,
                     checkPermutedFeatures=permute2Output)        
   }
   
   if(plotResults) {
      savemfrow = par("mfrow")        
      par(mfrow=c(2,2))
      
      if(check@checkType=="Classifier") {
         sigCheckPlot(output,classifier=TRUE,...)
      }
      
      if(!nosurv) {
         sigCheckPlot(output,classifier=FALSE,...)
      }
      
      par(mfrow=savemfrow)
   }
   
   return(output)
}

# Test Classification Potential of Gene Signature against Known Gene Signatures
# 
# Compares how an input gene signature classifies an expression set object. 
# Compares this classification using an SVM LOO CV analysis to known gene 
# signatures using the same classification scheme.
# Known gene signatures can either be passed in, or if not the package will 
# test against 48 known signatures from the v'ant Veer paper.

sigCheckKnown <- function(check, known="cancer"){
    
    bParallel <- TRUE
    
    
    expressionSet     <- check
    classes           <- check@classes
    signature         <- check@signature
    annotation        <- check@annotation 
    validationSamples <- check@validationSamples 
    survival          <- check@survival
    threshold         <- check@threshold
    nosecond          <- FALSE
    
    if(check@checkType=="Classifier") {
        classifier         <- check
        modeVal            <- check@modeVal
        Method             <- check@classifierMethod
        classifierScore    <- check@sigPerformance
        classifierSurvival <- check@survivalPval
        doMethod           <- .sigCheckKnownClassifier
        if(!length(validationSamples)) {
            bParallel <- FALSE
            nosecond  <- TRUE
            validationSamples <- xvalSpec("LOO")
        } 
        if(survival=="") {
            nosecond <- TRUE
        }
    } else if(check@checkType == "Survival") {
        Method           <- check@survivalMethod
        survivalLabel     <- check@survivalLabel
        timeLabel        <- check@timeLabel
        survivalPval     <- check@survivalPval
        doMethod         <- .sigCheckKnownSurvival
        if(!length(validationSamples)) {
            nosecond <- TRUE
        }
    } else {
        stop("Invalid SigCheck object.")
    }
    
    if(length(known)==1) {
        sigName <- known
        data(knownSignatures,envir=environment())
        sig <- names(knownSignatures) %in% sigName
        if(sum(sig)==0) {
            stop('Invalid known signature set: ',sigName)
        }
        known <- knownSignatures[[which(sig)]]
    } else {
        sigName <- "user specified"
    }
    
    signature <- .sigCheckSignature(expressionSet, signature, annotation)
    
    #Loop through Known Gene Signatures for Classification
    knownResult1 <- c(rep(0,length(known)))
    names(knownResult1) <- names(known)
    knownResult2 <- knownResult1
    totalSignatureGenesInEsets <- 0
    totalGenesInSignature <- 0
    
    if(bParallel) {
        knownList <- 
            bplapply(known, doMethod,
                     expressionSet, classes, validationSamples,
                     Method, annotation,
                     survival, threshold)
        if(nosecond){
            knownResult1 <- unlist(knownList)
        } else {
            knownResult1 <- sapply(knownList,function(x){x$result1})
            knownResult2 <- sapply(knownList,function(x){x$result2})
        }
    } else {
        knownResult1  <- NULL
        knownResult2  <- NULL
        for (i in 1:length(known)){
            checkit <- 
                doMethod(known[[i]],
                         expressionSet, classes, validationSamples,
                         Method, annotation,
                         survival, threshold)
            if(nosecond) {
                knownResult1 <- c(knownResult1, checkit)
            } else {
                knownResult1 <- c(knownResult1, checkit$result1)
                knownResult2 <- c(knownResult2, checkit$result2)
            }
        }
    }
    if(nosecond){
        names(knownResult1) = names(known)
    }
    output <- NULL
    if(check@checkType=="Classifier") {
        allScoreValues <- c(classifierScore, knownResult1)
        sortScores     <- sort(allScoreValues, decreasing=TRUE)
        geneListRank   <- mean(which(sortScores == classifierScore))
        checkPval      <- .sigCheckPval(classifierScore,knownResult1,lt=FALSE)
        nullPerf <- 
            .sigCheckClassifierNull(expressionSet, classes, check@validationSamples,
                                    modeVal=modeVal)
        output <- list(checkType="Known",
                       sigPerformance=classifierScore, 
                       modePerformance=nullPerf,
                       checkPval=checkPval,
                       performanceKnown=knownResult1)
    } else {
        allScoreValues       <- c(survivalPval,knownResult1)
        sortScores           <- sort(allScoreValues, decreasing=FALSE)
        geneListRank         <- mean(which(sortScores == survivalPval))
        geneListRank         <- mean(which(sortScores == survivalPval))
        checkPval            <- .sigCheckPval(survivalPval,knownResult1,lt=TRUE)
        output$checkType <- "Known"
        output$survivalPvalsKnown <- knownResult1
        output$survivalPval  <- survivalPval
        output$checkPval <- checkPval
    }
    
    #check matching features
    sigfeatures = NULL
    for(sig in known) {
        sigfeatures = unique(c(sigfeatures,sig))
    }
    matches <- sigfeatures %in% .sigCheckSignature(expressionSet, signature, 
                                                   annotation,
                                                   bReturnFeatures=TRUE)
    if(sum(matches) < length(sigfeatures)) {
        warning(
            sprintf("NOTE: %d unmatched features in known signatures (out of %d)",
                    length(sigfeatures)-sum(matches),length(sigfeatures)),
            call.=FALSE)   
    }
    
    #return
    output$known     <- sigName 
    output$knownSigs <- length(knownResult1)
    output$rank      <- geneListRank
    
    if(check@checkType=="Classifier") {
        if(!nosecond) {
            output$survivalPval  <- classifierSurvival
            output$survivalPvalsKnown <- knownResult2
        }
    } else {
        if(!nosecond) {
            output$trainingPvalsKnown <- knownResult2
        }
    }
    
    return(output)
}

.sigCheckKnownClassifier <- function(geneSig, expressionSet, 
                                     classes, validationSamples,
                                     Method, annotation, survival, threshold) {
    
    signature <- .sigCheckSignature(expressionSet,toupper(geneSig), annotation)
    #if no overlap
    if(length(signature) == 0){
        return(list(result1=NA,result2=NA)) 
    }
    expressionSet@signature <- signature
    classifier <- 
        sigCheckClassifier(expressionSet, 
                           plotTrainingKM=FALSE, plotValidationKM=FALSE)
    gc()
    if(survival=="") {
        return(result1=classifier@sigPerformance)        
    } else {
        return(list(result1=classifier@sigPerformance,
                    result2=classifier@survivalPval)) 
    }
    
}

.sigCheckKnownSurvival <- function(geneSig, expressionSet, 
                                   classes, validationSamples,
                                   Method, annotation, survival, threshold) {
    
    signature <- .sigCheckSignature(expressionSet,toupper(geneSig), annotation)
    #if no overlap
    if(length(signature) == 0){
        return(list(result1=NA,result2=NA)) 
    }
    expressionSet@signature <- signature
    check <- 
        suppressWarnings(
            sigCheckSurvival(expressionSet,
                             plotTrainingKM=FALSE, plotValidationKM=FALSE)
        )
    gc()
    if(is.null(validationSamples)) {
        return(result1=check@survivalPval)        
    } else {
        return(list(result1=check@survivalPval,
                    result2=check@survivalTrainingPval)) 
    }
    
}

# Test Classification Potential of Gene Signature
# 
# Compares how an input gene signature classifies an expression set object. 
# As default performs a leave
# one out analysis, if a validation set is not provided.

sigCheckClassifier <- function(SigCheck,  
                               plotTrainingKM=TRUE, plotValidationKM=TRUE,
                               impute=FALSE, ...){
    
    classifyFormula <- formula(sprintf("%s~.",SigCheck@classes))
    category <- which(varLabels(SigCheck) %in% SigCheck@classes)
    if(length(unique(SigCheck[[category]])) != 2) {
        stop('ERROR: classes can specify only two categories for classification.')
    }
    if(class(SigCheck[[category]])!="factor") {
        SigCheck[[category]] <- as.factor(SigCheck[[category]])
    }
    
    if (!length(SigCheck@validationSamples)){
        noValidation <- TRUE
        validationSamples <- xvalSpec("LOO")
        #Check to see if validation indices is already an xvalSpec object.
    } else {
        noValidation <- FALSE
        validationSamples <- SigCheck@validationSamples
    }
    
    if(class(validationSamples) == 'xvalSpec'){
        trainingSet <- validationSamples
    } else {
        totalNumSamples <- length(colnames(SigCheck))
        trainingSet <- 1:totalNumSamples
        trainingSet <- trainingSet[-validationSamples]
    }
    
    SigCheck <- .sigCheckNA(SigCheck,impute=impute)
    
    #Subset Whole Matrix Based Off of GeneSigIndices
    
    saveSig   <- SigCheck@signature
    signature <- .sigCheckSignature(SigCheck, SigCheck@signature, 
                                    SigCheck@annotation)
    
    subsetExpressionSet <- SigCheck[signature,]
    
    classifier <- 
        MLearn(classifyFormula, subsetExpressionSet, SigCheck@classifierMethod,
               trainInd=trainingSet,...)  
    classifierConfusionMatrix <- confuMat(classifier)
    classifierScore <- .sigCheckClassifyScore(classifier)
    
    if((SigCheck@survival!="")) {
        if(noValidation) {
            validationSamples=NULL
        }
        classifierSurvival <- 
            .sigCheckClassifierSurvival(expressionSet=SigCheck, 
                                        classes=SigCheck@classes,
                                        signature=signature,
                                        annotation=SigCheck@annotation,
                                        validationSamples=validationSamples,
                                        classifier=classifier,
                                        survival=SigCheck@survival,
                                        threshold=SigCheck@threshold,
                                        plotTrainingKM=plotTrainingKM,
                                        plotValidationKM=plotValidationKM,
                                        survivalLabel=SigCheck@survivalLabel, 
                                        timeLabel=SigCheck@timeLabel)
    }
    
    if(noValidation) {
        nullPerf <- 
            .sigCheckClassifierNull(SigCheck, SigCheck@classes, NULL,
                                    modeVal=SigCheck@modeVal)        
    } else {
        nullPerf <- 
            .sigCheckClassifierNull(SigCheck, SigCheck@classes, 
                                    validationSamples,modeVal=SigCheck@modeVal)
    }  
    
    SigCheck@checkType <- "Classifier"
    SigCheck@sigPerformance <- classifierScore
    SigCheck@confusion <- classifierConfusionMatrix
    SigCheck@modePerformance <- nullPerf
    SigCheck@classifier <- classifier
    SigCheck@signature <- saveSig
    
    if(SigCheck@survival!="") {
        SigCheck@survivalPval <- classifierSurvival$validationPval
        if(!missing(validationSamples)) {
            SigCheck@survivalTrainingPval <- classifierSurvival$trainPval
        }
    }
    return(SigCheck)
}

.sigCheckClassifyScore <- function(classOut) {
    given <- as.character(classOut@testOutcomes)
    predict <- as.character(classOut@testPredictions)
    correct <- sum(predict == given)
    score <- correct / length(given)
    return(score)    
}

.sigCheckClassifierNull <- function(expressionSet, classes, validationSamples,
                                    modeVal){
    if(modeVal=="") {
        modeVal=NULL
    }
    category <- which(varLabels(expressionSet) %in% classes)
    catVals <- unique(expressionSet[[category]])
    if(!length(validationSamples)){
        train <- expressionSet[[category]]
        valid <- expressionSet[[category]]
    } else {
        train <- expressionSet[[category]][-validationSamples]
        valid <- expressionSet[[category]][validationSamples]
    }
    if(is.null(modeVal)) {
        num1 <- sum(train %in% catVals[1])
        num2 <- sum(train %in% catVals[2])
        pick <- which.max(c(num1,num2))
        pickval <- catVals[pick]
    } else {
        if(modeVal %in% catVals) {
            pickval <- modeVal
        } else {
            stop("Mode value must be one of",catVals[1], "or", catVals[2])
        }
    }
    predict <- sum(valid %in% pickval)
    score <- predict/length(valid)
    return(score)
}

.sigCheckSignature <- function(expressionSet, signature, annotation,
                               bReturnFeatures=FALSE) {
    
    if(annotation=="") {
        annotation=NULL
    }
    if(is.null(annotation)) {
        features <- rownames(exprs(expressionSet))
    } else if(length(annotation)>1) {
        if(length(annotation != nrow(expressionSet))) {
            stop("Annotation doesn't have same number of labels as there are features",
                 call.=FALSE)
        } else {
            features <- annotation
        }
    } else {
        anno <- rownames(varMetadata(featureData(expressionSet))) %in% annotation
        if(sum(anno) == 0) {
            stop('Invalid annotation: ',annotation, call.=FALSE)
        }
        features <- featureData(expressionSet)[[which(anno)]]
    }
    if(bReturnFeatures) {
        return(features)
    }
    
    if(class(signature)=="integer") {
        if(max(signature)>length(features)) {
            stop('Invalid signature: feature number greater than number of features.',
                 call.=FALSE)
        }
    } else {
        matches <- signature %in% features
        if(sum(matches) < length(signature)) {
            warning(sprintf("NOTE: %d unmatched features in signature",
                            length(signature)-sum(matches)),call.=FALSE)       
        }
        signature <- which(features %in% signature)
    }
    return(signature)
}

.sigCheckNA <- function(expressionSet, impute=FALSE) {
   if(impute) {
      nas <- apply(exprs(expressionSet),1,function(x)sum(is.na(x)))
      if(sum(nas)) {
         message(sprintf("NOTE: %d values in %d features imputed (%2.4f%%).",
                      sum(nas),sum(nas>0),
                      (sum(nas)/(nrow(expressionSet)*ncol(expressionSet)))*100))      
         exprs(expressionSet) <- t(impute(t(exprs(expressionSet))))
      }
   } else {
      nas <- apply(exprs(expressionSet),1,function(x)sum(is.na(x))>0)
      if(sum(nas)) {
         warning(sprintf("NOTE: %d features with NA values removed",
                         sum(nas)),call.=FALSE)
      }
      expressionSet <- expressionSet[!nas,]
   }
   return(expressionSet)
}


.sigCheckClassifierSurvival <- function(expressionSet, classes, 
                                        signature, annotation, 
                                        validationSamples, 
                                        classifier, 
                                        survival, threshold=median, 
                                        plotTrainingKM, plotValidationKM,
                                        survivalLabel, timeLabel="Time"){
    
    expressionSet <- .sigCheckNA(expressionSet)
    
    signature <- .sigCheckSignature(expressionSet, signature, annotation)
    
    if(survivalLabel=="") {
        survivalLabel=survival
    }
    
    classes  <- .sigCheckPheno(expressionSet,classes,bAsFactor=TRUE)
    survival <- .sigCheckPheno(expressionSet,survival,bAsFactor=FALSE)
    
    #Get classification scores    
    savemfrow <- par("mfrow")
    if(length(validationSamples)) {
        par(mfrow=c(1,2))
    }
    
    if(!is.null(validationSamples)) {
        scores    <- trainScores(classifier)     
        trainPval <- .sigCheckPlotClassifierKM(expressionSet,classes,
                                               survival,scores,threshold,
                                               plotKM=plotTrainingKM,
                                               survivalLabel,timeLabel,
                                               main="Survival: Training Set")   
    } else {
        trainPval <- 1
    }
    
    scores <- testScores(classifier)
    validationPval <- .sigCheckPlotClassifierKM(expressionSet,classes,
                                                survival,scores,threshold,
                                                plotKM=plotValidationKM,
                                                survivalLabel,timeLabel,
                                                main="Survival: Validation Set")    
    
    par(mfrow=savemfrow)
    return(list(trainPval=trainPval,validationPval=validationPval))
    
}

.sigCheckPheno <- function(expressionSet,classes,bAsFactor=TRUE) {
    classnum <- which(varLabels(expressionSet) %in% classes)
    classvec <- expressionSet[[classnum]]
    if(bAsFactor) {
        if(class(classvec)!="factor") {
            classvec <- as.factor(classvec)
        }
    }
    return(classvec)
}

.sigCheckPlotClassifierKM <- function(expressionSet,classes,survival,
                                      scores,threshold,
                                      plotKM,survivalLabel,timeLabel,
                                      ...) {
    scorelabels <- rownames(scores)
    scores      <- scores[,1]
    
    validnames <- which(sampleNames(expressionSet) %in% scorelabels)
    classes <- classes[validnames]
    survival <- survival[validnames]
    
    if(is.function(threshold)) {
        threshold <- threshold(scores)    
    } else {
        threshold <- as.numeric(quantile(scores,threshold))
    }
    
    categories <- scores > threshold
    if(categories[1]) {
        l1 <- ">"; l2 <- "<"
    } else {
        l1 <- "<"; l2 <- ">"
    }
    if(length(unique(categories))==1){
        if(categories[1]) {
            warning("All scores above threshold.")
        } else {
            warning("All scores below threshold.")
        }
        return(1)
    }
    categories <- factor(categories,
                         labels=c(sprintf("%s%01.02f",l1,threshold), 
                                  sprintf("%s%01.02f",l2,threshold)))   
    
    pval <- .sigCheckPlotKM(survival, classes, categories, pvalOnly=!plotKM, 
                            xlab=timeLabel, 
                            ylab=survivalLabel, 
                            ...)
    return(pval)
}

# Test Classification Potential of Gene Signature against Random Gene Signatures
# 
# Compares how an input gene signature classifies an expression set object. 
# Compares this classification using an SVM LOO CV analysis to taking random 
# gene signatures of equal length 
# to the input list and testing classification power.

sigCheckRandom <- function(check, iterations=10){
    
    bParallel <- TRUE
    
    expressionSet     <- check
    classes           <- check@classes
    signature         <- check@signature
    annotation        <- check@annotation 
    validationSamples <- check@validationSamples 
    survival          <- check@survival
    threshold         <- check@threshold
    nosecond          <- FALSE
    
    if(check@checkType=="Classifier") {
        classifier         <- check
        modeVal            <- check@modeVal
        Method             <- check@classifierMethod
        classifierScore    <- check@sigPerformance
        classifierSurvival <- check@survivalPval
        doMethod           <- .sigCheckClassifierRandom
        if(!length(validationSamples)) {
            bParallel <- FALSE
            nosecond  <- TRUE
            validationSamples <- xvalSpec("LOO")
        } 
        if(survival=="") {
            nosecond <- TRUE
        }
    } else if(check@checkType == "Survival") {
        Method           <- check@survivalMethod
        survivalLabel     <- check@survivalLabel
        timeLabel        <- check@timeLabel
        survivalPval     <- check@survivalPval
        doMethod         <- .sigCheckSurvivalRandom
        if(!length(validationSamples)) {
            nosecond <- TRUE
        }
    } else {
        stop("Invalid SigCheck object.")
    }
    
    #expressionSet <- .sigCheckNA(expressionSet)
    
    signature <- .sigCheckSignature(expressionSet, signature, annotation)
    featNames <- .sigCheckSignature(expressionSet, signature, annotation,
                                    bReturnFeatures=TRUE)
    featNames <- featNames[!is.na(featNames)]
    
    #Random signature Testing
    sigLength <- length(signature)
    if(bParallel) {
        randomList <- bplapply(1:iterations, doMethod,
                               featNames, sigLength, expressionSet,
                               classes, validationSamples,
                               Method, survival, threshold)
        if(nosecond){
            randomResult  <- unlist(randomList)
        } else {
            randomResult  <- sapply(randomList,function(x){x$result1})
            randomResult2 <- sapply(randomList,function(x){x$result2})
        }
    } else {
        randomResult   <- NULL
        randomResult2 <- NULL
        for (i in 1:iterations){
            checkit <- 
                doMethod(i,featNames, sigLength, expressionSet, 
                         classes, validationSamples, 
                         Method, survival, threshold)
            if(nosecond) {
                randomResult <- c(randomResult, checkit)
            } else {
                randomResult  <- c(randomResult,  checkit$result1)
                randomResult2 <- c(randomResult2, checkit$result2)
            }
        }
    }
    
    output <- NULL
    if(check@checkType=="Classifier") {
        allScoreValues <- c(classifierScore, randomResult)
        sortScores     <- sort(allScoreValues, decreasing=TRUE)
        geneListRank   <- mean(which(sortScores == classifierScore))
        checkPval      <- .sigCheckPval(classifierScore,randomResult,lt=FALSE)
        nullPerf <- 
            .sigCheckClassifierNull(expressionSet, classes, check@validationSamples,
                                    modeVal=modeVal)
        output <- list(checkType="Random",
                       sigPerformance=classifierScore, 
                       modePerformance=nullPerf,
                       checkPval=checkPval,
                       performanceRandom=randomResult)
    } else {
        allScoreValues <- c(survivalPval,randomResult)
        sortScores     <- sort(allScoreValues, decreasing=FALSE)
        geneListRank   <- mean(which(sortScores == survivalPval))
        checkPval      <- .sigCheckPval(survivalPval,randomResult,lt=TRUE)
        output$checkType <- "Random"
        output$survivalPvalsRandom <- randomResult
        output$survivalPval <- survivalPval
        output$checkPval <- checkPval
    }
    
    output$tests <- iterations
    output$rank <- geneListRank
    
    if(check@checkType=="Classifier") {
        if(!nosecond) {
            output$survivalPval <- classifierSurvival
            output$survivalPvalsRandom <- randomResult2
        }
    } else {
        if(!nosecond) {
            output$trainingPvalsRandom <- randomResult2
        }
    }
    
    return(output)
}  

.sigCheckClassifierRandom <- function(i, featNames, sigLength, 
                                      expressionSet, classes,
                                      validationSamples, classifierMethod,
                                      survival=NULL, threshold) {
    expressionSet@signature <- sample(nrow(expressionSet), sigLength)
    classifier <- 
        suppressWarnings(
            sigCheckClassifier(expressionSet, 
                               plotTrainingKM=FALSE, plotValidationKM=FALSE)
        )
    gc()
    if(survival=="") {
        return(result1=classifier@sigPerformance)        
    } else {
        return(list(result1=classifier@sigPerformance,
                    result2=classifier@survivalPval)) 
    }
}

.sigCheckSurvivalRandom <- function(i, featNames, sigLength, 
                                    expressionSet, classes,
                                    validationSamples, survivalMethod,
                                    survival, threshold) {
    expressionSet@signature <- sample(nrow(expressionSet), sigLength)
    check <- 
        suppressWarnings(
            sigCheckSurvival(expressionSet, 
                             plotTrainingKM=FALSE, plotValidationKM=FALSE)
        )
    gc()
    if(!length(validationSamples)) {
        return(result1=check@survivalPval)        
    } else {
        return(list(result1=check@survivalPval,
                    result2=check@survivalTrainingPval)) 
    }
}

.sigCheckPval = function(performance,scores,lt=T) {
    if(lt) {
        pval <- 1 - (sum(performance < scores)/length(scores))
    } else {
        pval <- 1 - (sum(performance > scores)/length(scores))
    }
    accuracy <- ceiling(log10(length(scores)))
    pval = signif(pval,accuracy)
    return(pval)
}

%\VignetteIndexEntry{Checking gene expression signatures against random and known signatures with SigCheck}
%\VignettePackage{SigCheck}
%\VignetteEngine{utils::Sweave}

\documentclass{article}

<<style, eval=TRUE, echo=FALSE, results=tex>>=
BiocStyle::latex()
@ 

\newcommand{\reff}[1]{Figure \ref{fig:#1}}

\begin{document}
\SweaveOpts{concordance=TRUE}

\newcommand{\exitem}[3]{\item \texttt{\textbackslash#1\{#2\}} #3 \csname#1\endcsname{#2}.}
\title{Checking gene expression signatures against random and known signatures with \Biocpkg{SigCheck}}
\author{Rory Stark}

\date{Edited: December 9, 2015; Compiled: \today}

\maketitle

\tableofcontents

\section{Introduction}

A common task in the analysis of genomic data is the derivation of 
gene expression signatures that distinguish between phenotypes
(disease outcomes, molecular subtypes, etc.). 
However, in their paper "Most random gene expression signatures are 
significantly associated with breast cancer outcome" \cite{Venet2011}, 
Venet, Dumont, and Detour point out that while a gene signature may 
distinguish between two classes of phenotype, their ultimate uniqueness and 
utility may be limited. They show that while a specialized feature selection 
process may appear to determine a unique set of predictor genes, the resultant 
signature may not perform better than one made up of random genes, or genes 
selected at random from all differentially expressed genes. This suggests that 
the genes in the derived signature may not be particularly informative as to 
underlying biological mechanisms. 
They further show that gene sets that comprise published signatures for a wide 
variety of phenotypic classes may perform just as well at predicting arbitrary 
phenotypes; famously, they show that a gene signature that distinguishes 
postprandial laughter performs as well at predicting the outcome of breast
cancers as well as a widely-cited signature \cite{vantveer}.

The \Biocpkg{SigCheck} package was developed in order to make it easy to check 
a gene signature against random and known signatures, and assess the unique 
ability of that signature to distinguish phenotypical classes. 
It additionally provides the ability to check a signature's performance against
permuted data as a reality check that it is detecting a genuine signal 
in the original data. 
This vignette shows the process of performing the checks.

\section{Example dataset: NKI Breast Cancer Data and the van't Veer Signature}

In order to use \Biocpkg{SigCheck}, you must provide 
a) some data (an expression matrix), 
b) some metadata (feature names, survival data, class identifiers), 
and c) a gene signature (a subset of the features). 
For this vignette, we will use the NKI Breast Cancer dataset 
\Biocexptpkg{breastCancerNKI} and the associated van't Veer signature that 
predicts the likelihood that a patient will develop a distant metastases 
\cite{vantveer}. 

The dataset can be loaded as follows:
<<loadData, eval=TRUE, keep.source=TRUE>>=
library(breastCancerNKI)
library(Biobase)
data(nki)
nki
@

As can be seen, the \Rcode{nki} data is encapsulated in an 
\Robject{ExpressionSet} object. At its core, it contains an expression matrix 
consisting of 24,481 features (microarray probes mapped to genes) and 
337 samples (derived from tumor tissue taken from breast cancer patients). 
Included is phenotypical (clinical) metadata regarding the patients, 
including age of the patient, tumor grade, expression status of the 
ER, PR, and HER2 biomarkers, presence of a BRCA mutation, etc:

<<showPhenoData, eval=TRUE, keep.source=TRUE>>=
varLabels(nki)
@

The signatures derived by \cite{vantveer} and \cite{vandevijver} are used to 
predict the Distant Metastasis-Free Survival (DMFS) time. 
This is presented as a continuous measure indicating the time until the 
occurrence of a distant metastasis:

<<tDMFS, eval=TRUE, keep.source=TRUE>>=
nki$t.dmfs
@

DMFS is also presented as a binary class, with a cutoff used to distinguish 
between patients that had a recurrence event and those that did not:

<<eDMFS, eval=TRUE, keep.source=TRUE>>=
nki$e.dmfs
@

A number of patient samples do not 
have DMFS data available. 
Currently, \Biocpkg{SigCheck} can not handle NAs in the metadata, 
so the first step is to exclude these patients from
our analysis, which brings the number of samples down to 319:

<<excludeNA, eval=TRUE, keep.source=TRUE>>=
dim(nki)
nki <- nki[,!is.na(nki$e.dmfs)]
dim(nki)
@

The next step is to provide a gene signature to check. A core function of 
\Biocpkg{SigCheck} is to compare the performance of a gene signature with the 
performance of known gene signatures against the same data set. To accomplish 
this, it includes several sets of known signatures. One of these included 
signatures is the van't Veer signature, which we will use for this example.

To load the known gene signatures that are included with \Biocpkg{SigCheck}:

<<loadKnown, eval=TRUE, keep.source=TRUE>>=
library(SigCheck)
data(knownSignatures)
names(knownSignatures)
@

There are three sets of gene signatures, including a set of cancer signatures.
\footnote{See the Technical Notes section for information on how to obtain
more signature lists.} 
The van't Veer signature is one of the signatures in the known cancer 
gene signature set:

<<getVANTVEER, eval=TRUE, keep.source=TRUE>>=
names(knownSignatures$cancer)
vantveer <- knownSignatures$cancer$VANTVEER
vantveer
@

The signature is provided in the form of symbolic gene names. These will need 
to be matched up with the feature names in the \Robject{ExpressionSet}. 
While the default annotation is a probe identifier, the \Rcode{nki} dataset
provides a number of alternative annotations:

<<showFeatureAnno, eval=TRUE, keep.source=TRUE>>=
fvarLabels(nki)
@

We'll be using the \Rcode{"HUGO.gene-symbol"} to match the gene names in the 
van't Veer signature.

The final aspect of this dataset involve its division into 
a training (or discovery) set of samples and a validation set of samples. 
The training set should include all the samples
that were used in deriving the gene signature.
It is expected that the signature should perform optimally on these samples.
A validation n set of samples that played no role in deriving the signature is
required to test the efficacy of the signature.
For the NKI dataset, the \Rcode{varLabel} named \Rcode{"series"} specifies which 
samples were int he original trainingset, and which
were profiled in the follow-on experiment:
For this example, we will treat the first 100 samples as the training set.
<<showSeries, eval=TRUE, keep.source=TRUE>>=
table(nki$series)
@
\section{Example: Survival analysis}
In this section, we will use the survival data to test the ability of the
van't veer signature to predict outcome in the validation set. 
The most straightforward way to accomplish this involved two function calls. 
The first is a call to \Rfunction{sigCheck}, which sets up the experiment,
establishes the baseline performance, generates Kaplan-Meier plots, and
returns a newly constructed object of class \Robject{SigCheckObject}. 
The second call, to \Rfunction{sigCheckAll}, runs a default series
of tests before plotting and returning the results.

\subsection{Call: \Rfunction{sigCheck}}

The \Rfunction{sigCheck} function is the constructor 
for \Robject{SigCheckObject}s. 
It requires a number of parameters that are used to define the
experimental data and signature.

The first required parameter is \Rcode{expressionSet}, 
an \Robject{ExpressionSet} object that contains all the experimental data. 
In this example, \Rcode{expressionSet=nki}.
The next required parameter is \Rcode{classes}, a character string indicating
which of \Rcode{varLabels(expressionSet)} contains the binary class data. 
In this example, \Rcode{classes=e.dmfs}.

For a survival analysis, the \Rcode{survival} parameter indicates which 
 of \Rcode{varLabels(expressionSet)} contains the numeric survival data.
In this example, \Rcode{classes=t.dmfs}

The \Rcode{signature} parameter contains the signature that will be checked. 
This is most easily specified as a vector of feature names (ie gene IDs)
that match features in the \Rcode{annotation} parameter. 
The \Rcode{annotation} parameter specifies which of 
\Rcode{fvarLabels(expressionSet)} contains the feature labels.
In this example, \Rcode{signature=knownSignatures\$cancer\$VANTVEER}, and
\Rcode{annotation="HUGO.gene.symbol"}.

If the data are divided into training and validation samples, the samples 
that comprise the validation set should be specified using
the \Rcode{validationSamples} parameter.

The remaining parameters are optional, with usable defaults, and will be 
discussed in subsequent sections.

Putting this all together, the call to set up the experiment is:
<<sigCheckSurvival, fig=TRUE, include=FALSE, eval=TRUE, echo=TRUE,message=TRUE,warning=TRUE, width=10, height=8 >>=
check <- sigCheck(nki, classes="e.dmfs", survival="t.dmfs",
                  signature=knownSignatures$cancer$VANTVEER,
                  annotation="HUGO.gene.symbol",
                  validationSamples=which(nki$series=="NKI2")) 
check
@
\incfig{SigCheck-sigCheckSurvival}{.99\textwidth}{Results of 
\Rfunction{sigCheck} for NKI Breast Cancer dataset checking v'ant Veer 
signature.}

By default, this will generate two Kaplan-Meier plots, 
as shown in Figure~\ref{SigCheck-sigCheckSurvival}.
The first shows the
performance on the training set, and the other shows performance on the validation
set. The method that uses the signature to separate the samples
into two groups, based on taking the first principal component 
(as described in \cite{Venet2011}), is such that the High/Low groups can
be flipped in the two charts without concern. 

These plots show very strong performance on the training set, 
with distinct (but reduced) separation on the validation set.
Both p-values are reported as p<0.001.
Examination of the resulting object shows the actual p-value computed for the
validation set:

<<survivalPval, eval=TRUE, keep.source=TRUE>>=
check@survivalPval
@

\subsection{Call: \Rfunction{sigCheckAll}}

From this analysis, it appears that the signature does indeed have power in
distinguishing between two distinct survival groups. The next question is how
unique these specific genes in the signature are for this task. the function
\Rfunction{sigCheckAll} will run a series of checks for comparison purposes.
In each check, some background performance distribution is computed, and the 
performance of the signature is compared by calculating an empirical p-value.

The four tests include a distribution of p-values computed using randomly selected 
signature of the same size (number of features).
This check can be run separately using \Rfunction{sigCheckRandom}.
The second test compares the performance of this signature to a set of previously
identified ones. 
This check can be run separately using \Rfunction{sigCheckKnown}.
The third and fourth tests compare performance of the signature on the dataset
to permuted versions of the dataset; specifically permuting the \Rcode{class}
\Rcode{survival} metadata, as well as permuting the feature data by randomly 
re-assigning the expression values for each feature across the samples. 
The permuted data/metadata check can be run separately 
using \Rfunction{sigCheckPermuted}.

The parameters to \Rfunction{sigCheckAll} include the \Robject{SigCheckObject}
constructed by the previous call to \Rfunction{sigCheck}. 
The \Rcode{iterations} parameter determines the size of the null 
distribution for the random and permuted checks. 
The \Rcode{known} parameter specifies a set of known signatures to use
for the second check, with the default being the 48 cancer signatures 
identified by \cite{Venet2011}.

The number assigned to \Rcode{iterations} will determine the accuracy of the
p-value calculated for the random and permuted tests.
A value of at least 1000 is preferred to get a meaningful value:

<<sigCheckAllSurvivalNoeval, eval=FALSE, keep.source=TRUE>>=
nkiResults <- sigCheckAll(check, iterations=1000) 
@

As this vignette will take too long to run automatically with 
\Rcode{iterations=1000}, the results have been pre-computed and 
included as a data object
with the \Biocpkg(SigCheck) package:

<<loadResults, eval=TRUE, keep.source=TRUE>>=
data(nkiResults)
@

By default, \Rfunction{sigCheckAll} will generate plots of the results.
These can be re-generated using \Rfunction{sigCheckPlot}:
<<sigCheckPlotSurvival, fig=TRUE, include=FALSE, width=12, height=8 >>=
sigCheckPlot(nkiResults)
@
\incfig{SigCheck-sigCheckPlotSurvival}{.99\textwidth}{Results of 
\Rfunction{sigCheckPlot} for NKI Breast Cancer dataset checking v'ant Veer 
signature.}

In each of the four plots, the background distribution is plotted, with 
the x-axis representing survival p-values on a -log 10 scale, and the y-axis 
representing how many of the background tests produced p-values at that level.
The solid red vertical line shows the performance of the signature being tested.
The further to the right of the distribution this line is, the more uniquely
it performs for this check. In some cases, if the performance of the 
signature is out fo the range of the background distribution, 
this line will not be drawn. The vertical red dotted line shows where 
a "significant" result (p=0.05) would lie relative to
the background distribution.

The upper-left plot shows performance of the v'ant veer signature relative to 1,000
randomly chosen signature of comprised of an equal number of gene. The
upper-right plot shows the performance compared to 48 known
cancer prognostic signatures, while the two lower plots show performance
on permuted data (survival and gene expression across each feature).

The result returned by the call to \Rfunction{sigCheckAll} is a list of
results for each test:
<<sigCheckPlotSurvivalRes, eval=TRUE, keep.source=TRUE, echo=TRUE >>=
names(nkiResults)
@

The adjusted p-value computed for each check can be retrieved as follows:
<<sigCheckPlotSurvivalPvals, eval=TRUE, keep.source=TRUE, echo=TRUE >>=
nkiResults$checkRandom$checkPval
nkiResults$checkKnown$checkPval
nkiResults$checkPermutedSurvival$checkPval
nkiResults$checkPermutedFeatures$checkPval
@

\subsection{Scoring methods for dividing samples into survival groups}

The survival analysis depends on a method for using the signature to divide
the samples into survival groups before computing the p-value for how
their survival times differ. 
\Biocpkg{SigCheck} divides this task into two steps. 
The first is to calculate a score for each sample based on the 
feature values for each feature in the signature. The second step
is to divide the samples into groups based on these scores. 


\cite{Venet2011} discuss this issue,
and recommend first determining the score for each sample by
computing the first principal component of the expression matrix
(using only the features in the signature). 
They then suggest that the samples can be divided into two groups
based on whether their score is above or below the median score.
This is the default method used in \Biocpkg{SigCheck}.

There are options for both the scoring and dividing steps.
The scoring method is determined by the value of the parameter 
\Rcode{scoreMethod} for the function \Rfunction{sigCheck}.
The default value is \Rcode{"PCA1"}. 
\Rcode{scoreMethod} may also
be \Rcode{"High"}, which simply computes the mean expression value for 
each sample across the signature. 
This enables the samples to be divided into a "High" expression group
and a "Low" expression group.
Another option is to use a machine-learning classifier to do the scoring
(\Rcode{scoreMethod="classifier"}); the next section discusses how to
use classifiers in \Biocpkg{SigCheck}. 

Finally, \Rcode{scoreMethod} can be
set to a user-defined function if you want to do your own mapping of
expression value to scores. 
The user-defined scoring function should take an \Robject{ExpressionSet}
as a parameter.
This \Robject{ExpressionSet} will have as many rows as features that match
the signature, and as many columns as there are samples 
(either training or validation samples). 
It should return a vector of scores, one for each sample.

The second step is to use the scores to divide the samples into groups.
The \Rcode{threshold} parameter controls how this is accomplished.
Currently, the package only support division into either two or three groups, 
determined by which samples have scores below the specified percentiles, 
and which are greater than or equal to the specified percentile. 
The default is to use the median as the threshold, which will 
split the samples into two groups of approximately equal size.
In many cases, however, the real power of a signature is not to split 
the samples into two equal sized groups, but rather to
identify a subset of samples that have a distinct outcome.
Setting \Rcode{threshold=.66}, for example, will split the 
samples into a larger group with two-thirds of the sample, and
a smaller group containing the one-third of samples with the highest score.
If you are using a score such as \Rcode{scoreMethod="High"}, these would
be the samples with the highest mean expression over the signature, while
setting \Rcode{scoreMethod=.33} would split off a smaller group of
sample with the lowest mean expression.
By specifying two percentile cutoffs, you can split the samples into three groups:
one with high scores, one with low scores, and one with intermediate scores.
The performance of the signature will computed using only the samples in the
high and low groups. 

To see how all this works, consider some alternative scoring and grouping methods
for the sample data set. For example, we can see how the v'ant Veer signature 
performs when looking at overall expression levels. 
The code below takes advantage of the ability to create a new
\Robject{SigCheckObject} from an existing one:

<<sigCheckHi, fig=TRUE, include=FALSE, width=12, height=8 >>=
par(mfrow=c(2,2))
p5 <- sigCheck(check,
              scoreMethod="High",threshold=.5,
              plotTrainingKM=F)@survivalPval

p66 <- sigCheck(check,
              scoreMethod="High",threshold=.66,
              plotTrainingKM=F)@survivalPval

p33 <- sigCheck(check,
              scoreMethod="High",threshold=.33,
              plotTrainingKM=F)@survivalPval
p33.66 <-sigCheck(check,
              scoreMethod="High",threshold=c(.33,.66),
              plotTrainingKM=F)@survivalPval
p5
p66
p33
p33.66
@

\incfig{SigCheck-sigCheckHi}{.99\textwidth}{Results of 
\Rfunction{sigCheckPlot} for NKI Breast Cancer dataset checking v'ant Veer 
signature, using "High" scoring method and different percentile cutoffs.}

Figure~\ref{SigCheck-sigCheckHi} shows the baseline performance of the
signature using different thresholds. 
The one that splits off high and low expression groups, 
and eliminates the middle samples, performs the best. 
However to know how unique this performance is, the random and known checks
would have to be repeated using exactly the same evaluation criteria
as for the main signature.

\section{Example: Classification Analysis}

\Biocpkg{SigCheck} uses the \Biocpkg{MLInterfaces} package to enable a wide
range of machine-learning algorithms to be applied to expression data.
Classifiers constructed in this way use only the binary \Rcode{classes} metadata,
and not the survival data, to predict what class each sample belongs to.

Classifiers can be applied in two distinct ways: as a scoring method for
determining survival groups, and for assessing the classification potential
of signatures when survival data is unavailable.

\subsection{Classifiers for scoring survival groups}

As many classifiers generate a score for each sample 
(representing, for example, the probability that a sample belongs to 
a specific class), they can be used as the basis for dividing samples
into survival groups that can be uses to assess survival analysis performance.
This is accomplished by setting \Rcode{scoreMethod="classifier")} when
invoking \Rfunction({sigCheck}).

When specifying a classifier, the \Rcode{classifierMethod} parameter is
used to determine what type of classifier is to be used. 
This can be any classifier supported by the \Biocpkg{MLInterfaces} package.
The default, \Rcode{svmI}, uses a Support Vector Machine.
When invoked, the training set samples will be used to construct a classifier
that distinguished between the two classes specified in the 
\Rcode{classes} parameter.
When the training set is divided into survival groups, the resulting classifier
is used to generate scores for all the samples. These scores are then
subjected to the \Rcode{threshold} parameter as with the other
\Rcode{scoreMethod}s.

To see this in action with the sample dataset, all that is required is to
set \Rcode{scoreMethod="classifier"}:
<<sigCheckSurvivalClass, fig=TRUE, include=FALSE, eval=TRUE, echo=TRUE,message=TRUE,warning=TRUE, width=10, height=8 >>=
check <- sigCheck(check, scoreMethod="classifier") 
check@survivalPval
@
\incfig{SigCheck-sigCheckSurvivalClass}{.99\textwidth}{Results of 
\Rfunction{sigCheck} for NKI Breast Cancer dataset checking v'ant Veer 
signature, using a Support Vector Machine to classify samples into
survival groups.}

\subsection{Checking classifier performance independently of survival}
Another way of evaluating gene signatures is on how well they perform on 
the classification task itself.
For example, if the goals was to generate a gene signature to predict
susceptibility to a disease, the distinction between those who developed the 
disease and those who remain disease-free may be important than
the associated timeframes.
In some cases, only class data (recurrence/non recurrence; death/survival) may be
available for the samples, and no real-valued timescales.

In these cases (as well as cases where survival data are available but
classification  is of interest), the classification abilities of
a signature can be checked in similar manner to survival.

To generate such an analysis, simply leave the \Rcode{survival} unspecified:
<<sigCheckClassifier,include=FALSE,eval=TRUE,echo=TRUE,message=TRUE,warning=TRUE>>=
check <- sigCheck(nki, classes="e.dmfs",
                  signature=knownSignatures$cancer$VANTVEER,
                  annotation="HUGO.gene.symbol",
                  validationSamples=which(nki$series=="NKI2"),
                  scoreMethod="classifier") 
check
@

In this case, no Kaplan-Meier plots are generated as there is no survival data.
The baseline classification performance can be examined:

<<sigCheckClassifierStats, include=TRUE, eval=TRUE, echo=TRUE >>=
check@sigPerformance
check@modePerformance
check@confusion
@

The first value is the percentage of validation samples correctly classified. 
The second value is the percentage that would be correctly classified if
the classifier just guess the most frequently observed class in the
training set (the mode value).
The third value is the confusion matrix, which shows how validation samples
in each category were classified. 
This can be interpreted in terms of True Negatives and False Positives 
(first row), and False Negatives and True Positives (second row).

The classification performance of the signature can be checked against the 
performance of a background distribution of random or known signatures:

<<sigCheckClassify, eval=FALSE, keep.source=TRUE>>=
classifyRandom <- sigCheckRandom(check, iterations=1000)
classifyKnown <- sigCheckKnown(check)
@

As with the previous checks, the results have been pre-computed to
save computation time:
<<loadClassifyResults, eval=TRUE, keep.source=TRUE>>=
data(classifyResults)
@

The results can be plotted:
<<sigClassifyPlot, fig=TRUE, include=FALSE, width=12, height=6 >>=
par(mfrow=c(1,2))
sigCheckPlot(classifyRandom, classifier=TRUE)
sigCheckPlot(classifyKnown,  classifier=TRUE)
@
\incfig{SigCheck-sigClassifyPlot}{.99\textwidth}{Results of classifier 
performance check against random and known signature for 
NKI Breast Cancer dataset checking v'ant Veer 
signature, using "High" scoring method and different percentile cutoffs.}

Figure~\ref{SigCheck-sigClassifyPlot} shows the results of a classification 
analysis. 
The background distribution based on the classification performance
of the random or known signatures, so they are the percentage of validation
samples correctly classified.
That means that for these plots, better performing signatures are
toward the right on the x-axis.
Also, the dotted red vertical line represents the performance of a mode 
classifier. 
It can be very useful to see what a classifier that
always guesses the category more prevalent in the training set
for comparison to a curated gene signature and a sophisticated
machine learning algorithm.

\subsection{Classification without a validation set: leave-one-out cross-validation}

coming soon...

\section{Technical notes}

\subsection{Obtaining gene signatures from MSigDB}

When checking a signature's performance against known signatures 
(ie using the \Rfunction{sigCheckKnown} function), the gene
signatures available at the Broad Institute's \Biocexptpkg{MSigDB} site 
(part of \Biocexptpkg{GSEQ} \cite{MSigDB}). These are located at
the following URL: \url{http://software.broadinstitute.org/gsea/downloads.jsp}.
Note that you must first register and accept the licence terms,
which is why the signatures are not distributed with this package.

The gene sets can be downloaded in \Rcode{.gmt} format. 
This can be read in using the \Rfunction{read.gmt} function
from the \Biocpkg{qusage} package. 
For example, if you download the oncogenic signatures file, you
can retrieve the signatures as follows:

<<MSigDBNoeval, eval=FALSE, keep.source=TRUE>>=
c6.oncogenic <- read.gmt('c6.all.v5.0.symbols.gmt')
check.c6 <- sigCheckKnown(check, c6.oncogenic)
sigCheckPlot(check.c6)
@

\subsection{Use of \Biocpkg{BiocParallel} and \Rpackage{parallel}}

This note shows how to control the parallel processing in \Biocpkg{SigCheck}.

There are two different aspects of \Biocpkg{SigCheck} that are able to exploit
parallel processing. The primary one is when multiple signatures or datasets 
are being evaluated independently. This include the \Rcode{iterations} random
signatures in \Rfunction{sigCheckRandom}, the database of known signatures in 
\Rfunction{sigCheckKnown}, and the \Rcode{iterations} permuted datasets in 
\Rfunction{sigCheckPermuted}. In this case, the \Biocpkg{BiocParallel} package 
is used to carry out these comparisons in parallel. By default, 
\Biocpkg{BiocParallel} uses \Rpackage{parallel} to run in multicore mode, 
but it can also be configured to schedule a compute task across multiple 
computers. In the default multicore mode, it will use all of the cores on 
your system, which can result in a heavy load (especially if there is 
inadequate memory). You can manually set the number of cores to use as follows:


<<multicoreparam, eval=TRUE, keep.source=TRUE>>=
CoresToUse <- 6
library(BiocParallel)
mcp <- MulticoreParam(workers=CoresToUse)
register(mcp, default=TRUE)
@

which limits the number of cores in use at any one time to six. 
If you want to use only one core (serial mode), you can set 
\Rcode{CoresToUse <- 1}, or \Rcode{register(SerialParam())}.

The other aspect of processing that can use multiple processor cores is when
performing leave-one-out cross-validation (LOO-XV). In this case, the 
underlying \Biocpkg{MLInterfaces} package takes care of the parallelization
using the \Rpackage{parallel} package. You can set the number of cores that 
will be used for this as follows:

<<setcores, eval=TRUE, keep.source=TRUE>>=
options(mc.cores=CoresToUse)
@

Note that in the LOO-XV case, as every random or known signature, or permuted
dataset, requires parallel evaluation of cross-validated classifiers, the
parallelization at the level of \Rcode{iterations} is disabled automatically.

\section{Acknowledgements}

We would like to acknowledge everyone in the Bioinformatics Core at 
Cancer Research UK's Cambridge Institute at the University of Cambridge, 
as well as members of the Ponder group (particularly Kerstin Meyer), 
for their support and contributions.

\section{Session Info}
<<sessionInfo, results=tex, print=TRUE, eval=TRUE>>=
toLatex(sessionInfo())
@

\bibliography{SigCheck}

\end{document}

\name{sigCheck}
\alias{sigCheck}

\title{
Create a \code{\link{SigCheckObject}} and establish baseline performance.
}

\description{
Main constructor for a \code{\link{SigCheckObject}}. Also establishes 
baseline survival analysis and/or classification performance.
}

\usage{
sigCheck(expressionSet, classes, survival, signature, 
         annotation, validationSamples, 
         scoreMethod="PCA1", threshold=median,
         classifierMethod=svmI, modeVal,
         survivalLabel, timeLabel,
         plotTrainingKM=TRUE, plotValidationKM=TRUE,
         impute=TRUE)
}


\arguments{

\item{expressionSet}{
An \code{\link{ExpressionSet}} object containing the data to be checked, 
including an expression matrix, feature labels, and samples.

\code{expressionSet} can also be an existing \code{\link{SigCheckObject}},
in which case everything will be inherited from the passed object 
except the values for any specified parameters,
}

\item{classes}{
Specifies which label is to be used to determine the prognostic categories 
(must be one of \code{varLabels(expressionSet))}. 
There should be only 
two unique values in expressionSet$classes.
}

\item{survival}{
Specifies which label is to be used to determine survival times.
(must be one of \code{varLabels(expressionSet))}.
This may be missing if only classification is being checked.
}

\item{signature}{
A vector of feature labels specifying which features comprise the signature to 
be checked. These feature labels should match values as specified in the 
\code{annotation} parameter. 
Alternatively, this can be a integer vector of feature indexes.
}

\item{annotation}{
Character string specifying which \code{\link{fvarLabels}} field should be 
used as the annotation. If missing, the row names of the \code{expressionSet} 
are used as the feature names.
}

\item{validationSamples}{
Optional specification, as a vector of sample indices, of what samples in the 
\code{expressionSet} should be considered validation samples.
If present, the main checks will be performed using only these samples.
If a the \code{scoreMethod} parameter is equal to \code{"classifier"},
the remaining samples will 
be used as a training set to construct a classifier that will be used to 
separate the training samples.
If a classifier is used, and \code{validationSamples} is not specified,
a leave-one-out (LOO) validation
method will be used, where a separate classifier will be trained to classify 
each sample using the all the remaining samples.
}

\item{scoreMethod}{
specification of how the samples should be split into groups for 
survival analysis. If a character sting, one of the following values:
\itemize{ 
\item \code{"PCA1"}: default scoring method for separating validation
samples into groups
by taking the value of the first principal component of the expression
values in the signature for each sample.
\item \code{"High"}: score used for separating validation samples into groups
for each sample is the mean value over 
all the expression values in the signature for each sample.
\item \code{"classifer"}: score used for separating validation 
samples into groups is determined by a classifier specified in the
\code{classifierMethod} parameter. If the \code{survival} parameter is 
specified, the classifier method must return a real-valued score for each
predicted sample.
}

\code{scoreMethod} can also be a user-defined function that computes a score.
The function should take a single parameter, an \code{\link{ExpressionSet}}, and
return a vector of score, one for each row.

if the \code{survival} parameter is missing, \code{scoreMethod} value must be 
\code{"classifier"}.
}
\item{threshold}{
specifies the threshold used for separating the validation samples into classed
based on the score derived using \code{scoreMethod}. 
Can be either a function, (with default \code{median}) or a number between 
zero and one indicating a percentile. Validation samples will be divided into
a group whose percentile scores are less than this value, and another 
group with percentile scores greater 
than or equal to this value.
\code{threshold} may also be a vector of two percentiles,
in which case samples
will be divided into High, Low, and Mid groups.
The survival p-value will be computed using only the high and low
groups, with the mid group samples excluded.
}

  \item{classifierMethod}{
  if the \code{scoreMethod} is equal to \code{"classifer"}, this specifies 
  what classifier to use. It is a \code{MLInterfaces} \code{learnerSchema}
  object indicating the machine learning method 
to use for classification. Default is \code{\link{svmI}} for linear 
Support Vector Machine classification.  See \code{\link{MLearn}} for
available methods.
}
  \item{modeVal}{
specifies which of the two category values (one of the values implied 
by the \code{classes} parameter) should be considered as the default
value when computing the performance of a "mode" classifier. Is missing,
the actual mode (most commonly occurring) value of the training set will be used.
}

\item{survivalLabel}{
String to use in the Y-axis of any Kaplan-Meier plots generated, this indicates
what aspect of survival is being predicted, such as time to recurrence or death.
}
\item{timeLabel}{
String to use in the X-axis of an Kaplan-Meier plots generated, this indicates
the units of time, such as days or months to outcome event.
}
\item{plotTrainingKM}{
if the \code{survival} and \code{validationSamples} parameters are provided,
a Kaplan-Meier plot can be plotted automatically for the training set samples if
this is \code{TRUE}. A value of \code{FALSE} will suppress the plot being
automatically generated.
}
\item{plotValidationKM}{
if the \code{survival} parameter is provided,
a Kaplan-Meier plot can be plotted automatically for the validation set samples if
this is \code{TRUE}. A value of \code{FALSE} will suppress the plot being
automatically generated.
Note that is the \code{validationSamples} parameter is missing, the resulting
plot will be over all samples.
}
\item{impute}{
if \code{TRUE}, missing data values in the \code{expressionSet} will be imputed.
If \code{FALSE}, any features with any missing values will be removed 
from the dataset.
}
}

\details{
This function constructs a new \code{\link{SigCheckObject}} and carried out
a baseline analysis, 
which will vary depending on which parameters are specified.

If the \code{survival} parameter is specified, a survival analysis
is carried out. 
If the \code{validationSamples} parameter is specified, this will be done 
separately on the validation samples and the remaining 
(training/discovery) samples. 
The main result is a p-value indicating the confidence that the samples are
separable into groups with distinct survival outcomes. This value is obtained
using the \code{\link{survdiff}} function in the \code{survival} package 
(and applying \code{\link{pchisq}} to the 
\code{$chisq} component of the result). The samples are separated into groups
using the \code{scoreMethod} and \code{threshold} parameters 
(and possibly the \code{classifierMethod} parameter).

If the \code{survival} parameter is not specified, then the \code{scoreMethod} 
parameter must be equal to \code{"classifier"}, and a pure classification 
analysis is completed (as was done in \code{SigCheck 1.0}). 
If the \code{validationSamples} parameter is specified, the remaining samples 
are used as a training set to construct a classifier that is used
to classify the validation samples. If \code{validationSamples} is not
specified, leave-one-out cross-validation is used whereby a separate
classifier is trained to predict each sample using all of the others.

}
\value{
If the baseline analysis can be completed, 
a \code{\link{SigCheckObject}} is returned.
}

\author{
Rory Stark with Justin Norden
}

\seealso{
\code{\link{sigCheckAll}}, \code{\link{sigCheckRandom}}, 
 \code{\link{sigCheckKnown}}, \code{\link{sigCheckPermuted}}.
}
\examples{
library(breastCancerNKI)
data(nki)
nki <- nki[,!is.na(nki$e.dmfs)]
data(knownSignatures)

## survival analysis 
check <- sigCheck(nki, classes="e.dmfs", survival="t.dmfs",
                  signature=knownSignatures$cancer$VANTVEER,
                  annotation="HUGO.gene.symbol") 
check@survivalPval
check <- sigCheck(check, classes="e.dmfs", survival="t.dmfs",
                  signature=knownSignatures$cancer$VANTVEER,
                  annotation="HUGO.gene.symbol",
                  scoreMethod="High", threshold=.33) 
check@survivalPval

## survival analysis with separate training and validation using SVM
check <- sigCheck(nki, classes="e.dmfs", survival="t.dmfs",
                  signature=knownSignatures$cancer$VANTVEER,
                  annotation="HUGO.gene.symbol",
                  validationSamples=150:319,
                  scoreMethod="classifier") 
check
}

\name{SigCheckObject-class}
\Rdversion{1.1}
\docType{class}
\alias{SigCheckObject-class}
\alias{SigCheckObject}
%% \alias{annotation,SigCheckObject-method}
%% \alias{checkType,SigCheckObject-method}
%% \alias{classes,SigCheckObject-method}
%% \alias{classifier,SigCheckObject-method}
%% \alias{classifierMethod,SigCheckObject-method}
%% \alias{confusion,SigCheckObject-method}
%% \alias{modePerformance,SigCheckObject-method}
%% \alias{modeVal,SigCheckObject-method}
%% \alias{signature,SigCheckObject-method}
%% \alias{sigPerformance,SigCheckObject-method}
%% \alias{survival,SigCheckObject-method}
%% \alias{survivalClassificationScore,SigCheckObject-method}
%% \alias{survivalConfusionMatrix,SigCheckObject-method}
%% \alias{survivalMethod,SigCheckObject-method}
%% \alias{survivalPval,SigCheckObject-method}
%% \alias{survivalScores,SigCheckObject-method}
%% \alias{survivalTrainingClassificationScore,SigCheckObject-method}
%% \alias{survivalTrainingConfusionMatrix,SigCheckObject-method}
%% \alias{survivalTrainingPval,SigCheckObject-method}
%% \alias{survivalTrainingScores,SigCheckObject-method}
%% \alias{survivalType,SigCheckObject-method}
%% \alias{threshold,SigCheckObject-method}
%% \alias{timeUnits,SigCheckObject-method}
%% \alias{validationSamples,SigCheckObject-method}

\title{Class \code{"SigCheckObject"}}
\description{
The main object containing everything associated with an expression
dataset and a gene signatures.
Used for subsequent checks of the unique prognostic and/or classification performance of the signature.
Based on an \code{\link{ExpressionSet}} object.
}
\section{Objects from the Class}{
The preferred way to create a \code{\link{SigCheckObject}} is to use the 
\code{\link{sigCheck}} function.
}
\section{Slots}{
  \describe{
    \item{\code{checkType}:}{Object of class \code{"character"} ~~ }
    \item{\code{classes}:}{Object of class \code{"character"} ~~ }
    \item{\code{annotation}:}{Object of class \code{"character"} ~~ }
    \item{\code{survival}:}{Object of class \code{"character"} ~~ }
    \item{\code{signature}:}{Object of class \code{"vector"} ~~ }
    \item{\code{signatureLabels}:}{Object of class \code{"vector"} ~~ }
    \item{\code{validationSamples}:}{Object of class \code{"vector"} ~~ }
    \item{\code{survivalMethod}:}{Object of class \code{"character"} ~~ }
    \item{\code{threshold}:}{Object of class \code{"ANY"} ~~ }
    \item{\code{survivalScores}:}{Object of class \code{"numeric"} ~~ }
    \item{\code{survivalConfusionMatrix}:}{Object of class \code{"matrix"} ~~ }
    \item{\code{survivalClassificationScore}:}{Object of class \code{"numeric"} ~~ }
    \item{
       \code{survivalPval}:}{Object of class \code{"numeric"}, representing
       performance of the signature in dividing the samples into sets with
       distinct survival prognosis.  
    }
    \item{\code{survivalTrainingScores}:}{Object of class \code{"numeric"} ~~ }
    \item{\code{survivalTrainingConfusionMatrix}:}{Object of class \code{"matrix"} ~~ }
    \item{\code{survivalTrainingClassificationScore}:}{Object of class \code{"numeric"} ~~ }
    \item{\code{survivalTrainingPval}:}{Object of class \code{"numeric"} ~~ }
    \item{\code{survivalLabel}:}{Object of class \code{"character"} ~~ }
    \item{\code{timeLabel}:}{Object of class \code{"character"} ~~ }
    \item{\code{classifierMethod}:}{Object of class \code{"learnerSchema"} ~~ }
    \item{\code{sigPerformance}:}{Object of class \code{"numeric"} ~~ }
    \item{\code{confusion}:}{Object of class \code{"matrix"} ~~ }
    \item{\code{modeVal}:}{Object of class \code{"character"} ~~ }
    \item{\code{modePerformance}:}{Object of class \code{"numeric"} ~~ }
    \item{\code{classifier}:}{Object of class \code{"classifierOutput"} ~~ }
    \item{\code{experimentData}:}{Object of class \code{"MIAME"} ~~ }
    \item{\code{assayData}:}{Object of class \code{"AssayData"} ~~ }
    \item{\code{phenoData}:}{Object of class \code{"AnnotatedDataFrame"} ~~ }
    \item{\code{featureData}:}{Object of class \code{"AnnotatedDataFrame"} ~~ }
    \item{\code{protocolData}:}{Object of class \code{"AnnotatedDataFrame"} ~~ }
    \item{\code{.__classVersion__}:}{Object of class \code{"Versions"} ~~ }
  }
}
\section{Extends}{
Class \code{"\linkS4class{ExpressionSet}"}, directly.
Class \code{"\linkS4class{eSet}"}, by class "ExpressionSet", distance 2.
Class \code{"\linkS4class{VersionedBiobase}"}, by class "ExpressionSet", distance 3.
Class \code{"\linkS4class{Versioned}"}, by class "ExpressionSet", distance 4.
}
\section{Methods}{
%%   \describe{
%%     \item{annotation}{\code{signature(object = "SigCheckObject")}: ... }
%%     \item{checkType}{\code{signature(object = "SigCheckObject")}: ... }
%%     \item{classes}{\code{signature(object = "SigCheckObject")}: ... }
%%     \item{classifier}{\code{signature(object = "SigCheckObject")}: ... }
%%     \item{classifierMethod}{\code{signature(object = "SigCheckObject")}: ... }
%%     \item{confusion}{\code{signature(object = "SigCheckObject")}: ... }
%%     \item{modePerformance}{\code{signature(object = "SigCheckObject")}: ... %% }
%%     \item{modeVal}{\code{signature(object = "SigCheckObject")}: ... }
%%     \item{signature}{\code{signature(object = "SigCheckObject")}: ... }
%%     \item{sigPerformance}{\code{signature(object = "SigCheckObject")}: ... }
%%     \item{survival}{\code{signature(object = "SigCheckObject")}: ... }
%%     \item{survivalClassificationScore}{\code{signature(object = "SigCheckObject")}: ... }
%%     \item{survivalConfusionMatrix}{\code{signature(object = "SigCheckObject")}: ... }
%%     \item{survivalMethod}{\code{signature(object = "SigCheckObject")}: ... }
%%     \item{survivalPval}{
%%        \code{signature(object = "SigCheckObject")}: retrieves value of 
%%        \code{survivalPval} slot.
%%     }
%%     \item{survivalScores}{\code{signature(object = "SigCheckObject")}: ... }
%%     \item{survivalTrainingClassificationScore}{\code{signature(object = "SigCheckObject")}: ... }
%%     \item{survivalTrainingConfusionMatrix}{\code{signature(object = "SigCheckObject")}: ... }
%%     \item{survivalTrainingPval}{\code{signature(object = "SigCheckObject")}: ... }
%%     \item{survivalTrainingScores}{\code{signature(object = "SigCheckObject")}: ... }
%%     \item{survivalType}{\code{signature(object = "SigCheckObject")}: ... }
%%     \item{threshold}{\code{signature(object = "SigCheckObject")}: ... }
%%     \item{timeUnits}{\code{signature(object = "SigCheckObject")}: ... }
%%     \item{validationSamples}{\code{signature(object = "SigCheckObject")}: ... }
%% 	 }

No public methods, access slots directly if required. 
}
%% \references{
%%  ~~put references to the literature/web site here~~
%% }
\author{
Rory Stark
}
\note{
More methods and documentation coming soon...
}



\seealso{
\code{\link{sigCheck}}
}
\examples{
showClass("SigCheckObject")
}
\keyword{classes}

\name{sigCheckRandom}
\alias{sigCheckRandom}
\title{
Check signature performance against signatures composed of randomly selected 
features
}
\description{
Performance of a signature is compared to performance of signatures composed of 
the same number of randomly-selected features.
}
\usage{
sigCheckRandom(check, iterations=10)
}

\arguments{
\item{check}{
A \code{\link{SigCheckObject}}, as returned by \code{\link{sigCheck}}.
}
  \item{iterations}{
The number of random signatures the primary signature will be compared to.
This should be at least 1,000 to compute a meaningful empirical p-value
for comparative performance.
}
}
\details{
\cite{sigCheckRandom} will select \code{iterations} signatures, each consisting
of the same number of features as are in the primary signature
provided in the call to \code{\link{sigCheck}} that created the 
\code{\link{SigCheckObject}} sampled at random from all available features.

Each random signature will be evaluated in the same manner as the primary
signature.
If survival data were supplied, a survival analysis will be carried out on the 
validation samples, and a p-value computed as a performance measure. 
If no survival data are available, the training samples will be used to train
a classifier, and the performance score will be percentage of validation samples
correctly classified. (If no validation samples are provided, leave-one-out
cross validation will be used to calculate the classification performance 
for each random signature).

An empirical p-value will be computed based on the percentile rank of the 
performance of the primary signature compared to a null distribution of
the performance of the random signatures.
}
\value{
A result list with the following elements:

\itemize{

\item \code{$checkType} is equal to \code{"Random"}.

\item \code{$tests} is the number of tests run (equal to \code{iterations}.)

\item \code{$rank} is the performance rank of the primary signature  
within the performance of the random signatures.

\item \code{$checkPval} is the empirical p-value computed using the scores
of the random signature as a null distribution. A value of zero indicates that
no random signatures performed as good or better than the primary signature.

\item \code{$survivalPval} represents the performance of the primary,
if survival data were provided.

\item \code{$survivalPvalsRandom} is a vector of performance scores (p-values) 
for each random signature on the validation samples, if survival data
were provided.

\item \code{$trainingPvalsRandom} is a vector of performance scores (p-values) 
for each random signature on the training samples, if survival data
and separate validation samples were provided.

\item \code{$sigPerformance} is the proportion of validation samples 
correctly classified by the primary signature if a classifier was used.

\item \code{$modePerformance} is the proportion of validation samples 
correctly classified using a mode classifier.

\item \code{$performanceRandom} is a vector of classification performance 
scores for each random signature, each indicating the proportion
of validation samples correctly classified if a classifier was used.
}
}

\author{
Rory Stark
}

\seealso{
\code{\link{sigCheck}}, \code{\link{sigCheckAll}}, 
\code{\link{sigCheckPermuted}}, \code{\link{sigCheckKnown}},
\code{\link{sigCheckPlot}}
}
\examples{
#Disable parallel so Bioconductor build won't hang
library(BiocParallel)
register(SerialParam())

library(breastCancerNKI)
data(nki)
nki <- nki[,!is.na(nki$e.dmfs)]
data(knownSignatures)
ITERATIONS <- 5 # should be at least 20, 1000 for real checks

## survival analysis
check <- sigCheck(nki, classes="e.dmfs", survival="t.dmfs",
                  signature=knownSignatures$cancer$VANTVEER,
                  annotation="HUGO.gene.symbol",
                  validationSamples=150:319)

randomResult <- sigCheckRandom(check, iterations=ITERATIONS)
randomResult$checkPval
sigCheckPlot(randomResult)
}

\name{sigCheckKnown}
\alias{sigCheckKnown}
\title{
Check signature performance against a panel of known
signatures.
}
\description{
Performance of a  signature is compared to performance of
a panel of
known (previously identified) signature.
}
\usage{
sigCheckKnown(check, known="cancer")
}

\arguments{
\item{check}{
A \code{\link{SigCheckObject}}, as returned by \code{\link{sigCheck}}.
}
  \item{known}{
Either a character string specifying which set of signatures to use from the 
included sets in \code{\link{knownSignatures}}, 
or a list of previously identified signatures to compare performance against. 
Each element in the list should be a vector of feature labels. 
Default is to use the "cancer" signatures from the included k
\code{\link{knownSignatures}} data set, taken from Venet et. al.
}
}

\details{
Each specified known signature will be evaluated in the 
same manner as the primary signature.
If survival data were supplied, a survival analysis will be carried out on the 
validation samples, and a p-value computed as a performance measure. 
If no survival data are available, the training samples will be used to train
a classifier, and the performance score will be percentage of validation samples
correctly classified. (If no validation samples are provided, leave-one-out
cross validation will be used to calculate the classification performance 
for each known signature).

An empirical p-value will be computed based on the percentile rank of the 
performance of the primary signature compared to a null distribution of
the performance of the known signatures.
}

\value{
A result list with the following elements:

\itemize{

\item \code{$checkType} is equal to \code{"Known"}.

\item \code{$knownSigs} is the number of tests run (equal to the
number of known signatures indicated where at least one gene matches
a feature).

\item \code{$rank} is the performance rank of the primary signature  
within the performance of the known signatures.

\item \code{$checkPval} is the empirical p-value computed using the scores
of the known signature as a null distribution. A value of zero indicates that
no known signatures performed as good or better than the primary signature.

\item \code{$survivalPval} represents the performance of the primary signature,
if survival data were provided.

\item \code{$survivalPvalsKnown} is a vector of performance scores (p-values) 
for each known signature on the validation samples, if survival data
were provided.

\item \code{$trainingPvalsKnown} is a vector of performance scores (p-values) 
for each known signature on the training samples, if survival data
and separate validation samples were provided.

\item \code{$sigPerformance} is the proportion of validation samples 
correctly classified by the primary signature if a classifier was used.

\item \code{$modePerformance} is the proportion of validation samples 
correctly classified using a mode classifier.

\item \code{$performanceKnown} is a vector of classification performance
scores for each
known signature, each indicating the proportion
of validation samples correctly classified is a classifier was used.
}
}

\references{
Venet, David, Jacques E. Dumont, and Vincent Detours. 
"Most random gene expression signatures are significantly associated 
with breast cancer outcome." PLoS Computational Biology 7.10 (2011): e1002240.
}

\author{
Rory Stark
}

\seealso{
\code{\link{knownSignatures}}, \code{\link{sigCheck}}, 
\code{\link{sigCheckAll}}, \code{\link{sigCheckRandom}}, 
\code{\link{sigCheckPermuted}}, \code{\link{sigCheckPlot}}
}
\examples{
#Disable parallel so Bioconductor build won't hang
library(BiocParallel)
register(SerialParam())

library(breastCancerNKI)
data(nki)
nki <- nki[,!is.na(nki$e.dmfs)]
data(knownSignatures)

## survival analysis
check <- sigCheck(nki, classes="e.dmfs", survival="t.dmfs",
                  signature=knownSignatures$cancer$VANTVEER,
                  annotation="HUGO.gene.symbol",
                  validationSamples=150:319)

knownResult <- sigCheckKnown(check)
knownResult$checkPval
knownResult$survivalPvalsKnown[knownResult$survivalPvalsKnown <
                               knownResult$checkPval]
sigCheckPlot(knownResult)
}

\name{SigCheck-package}
\alias{SigCheck-package}
\alias{SigCheck}
\docType{package}
\title{
Check a gene signature's survival and/or classification performance against 
random signatures, known signatures, and permuted data/metadata.
}
\description{
While gene signatures are frequently used to predict phenotypes
        (e.g. predict prognosis of cancer patients), it it not always
        clear how optimal or meaningful they are (cf David Venet,
        Jacques E. Dumont, and Vincent Detours' paper "Most Random Gene
        Expression Signatures Are Significantly Associated with Breast
        Cancer Outcome"). Based on suggestions in that paper,
        SigCheck accepts a data set (as an ExpressionSet) and a gene
        signature, and compares its performance on survival and/or 
        classification tasks against 
        a) random gene signatures of the same length; 
        b) known, related and unrelated gene signatures; 
        and c) permuted data and/or metadata.
}
\details{
\tabular{ll}{
Package: \tab SigCheck\cr
Type: \tab Package\cr
Version: \tab 2.0\cr
Date: \tab 2015-05-18\cr
License: \tab Artistic-2.0\cr
}
To use \code{SigCheck}, first create anew SigCheck object using the 
function \code{\link{sigCheck}}. 
This will establish the baseline performance
of the signature. 
Next, either run specific checks, or use the high level function 
\code{\link{sigCheckAll}} to run all the core functions in turn.
The three core functions enable 
1) comparison of baseline performance against signatures 
composed of random genes (\code{\link{sigCheckRandom}});
2) comparison of baseline performance against known, and mostly unrelated, 
gene signatures (\code{\link{sigCheckKnown}}); and 
3) comparison of baseline performance against randomly
permuted data and/or metadata (\code{\link{sigCheckPermuted}}).

At a minimum, SigCheck requires a data set (as an \code{\link{ExpressionSet}}),
metadata indicating the membership of each sample in one of two classes,
and a signature (a subset of features in the ExpressionSet). 
If survival data are available, survival analyses are carried out.
Validation samples can be divided into two classes using one of the simple
default methods (based on overall expression value 
or their first principal component).
Alternatively, more sophisticated classification algorithms can be
deployed, using the
\code{\link{MLearn}} function from the \code{MLInterfaces} package 
to build a classifier (using \code{link{smvI}} by default). 
If no validation samples are 
specified, leave-one-out (LOO) cross-validation is utilized to build multiple 
classifiers, each predicting one sample.
If no survival data are provided, signatures are evaluated
based on classification performance.

Output of each check includes the distribution of random performance scores 
(either survival p-value or classification performance)
and the ranking of the passed signature within this distribution. 
An empirical p-value calculation based on 
this rank is also returned indicating confidence that the performance of 
the signature being checked has unique power.
}
\author{
First version written by Justin Norden with Rory Stark at the University of 
Cambridge, Cancer Resaerch UK Cambridge Institute.

Second version, including all survival analysis, written by Rory Stark at CRUK-CI.

Maintainer: Rory Stark <rory.stark@cruk.cam.ac.uk>

}
\references{
Venet, David, Jacques E. Dumont, and Vincent Detours. 
"Most random gene expression signatures are significantly associated with 
breast cancer outcome." PLoS Computational Biology 7.10 (2011): e1002240.
}

\keyword{package}

\name{sigCheckPermuted}
\alias{sigCheckPermuted}
\title{
Check signature performance against performance on randomly permuted data.
}
\description{
Performance of a  signature on intact data is compared to
performance on permuted data and/or metadata. Data may be permuted by
by feature (expression values of each 
feature permuted across samples), samples (expression values of all features 
permuted within each sample). Metadata may be permuted by categories 
(permuted assignment of samples to 
classification categories) or survival (permuted assignment
of survival times to samples).
}
\usage{
sigCheckPermuted(check, toPermute="categories", iterations=10)
}


\arguments{
\item{check}{
A \code{\link{SigCheckObject}}, as returned by \code{\link{sigCheck}}.
}

  \item{toPermute}{
Character string or vector of strings indicating what should be permuted. 
Allowable values:
\itemize{
\item \code{"features"}: the expression values for each feature will be 
permuted (permutation by row).

\item \code{"samples"}: the expression values for each sample will be 
permuted (permutation by column).

\item \code{"survival"}: the values in \code{survival} will be permuted.

\item \code{"categories"}: the values in \code{classes} will be permuted.
}}

  \item{iterations}{
The number of permuted dataset the primary signature will be compared to.
This should be at least 1,000 to compute a meaningful empirical p-value
for comparative performance.
}
}

 
\details{
The primary signature  will be evaluated against each permuted dataset in the 
same manner as for the intact dataset.

If survival data were supplied, a survival analysis will be carried out on the 
validation samples, and a p-value computed as a performance measure. 
If no survival data are available, the training samples will be used to train
a classifier, and the performance score will be percentage of validation samples
correctly classified. (If no validation samples are provided, leave-one-out
cross validation will be used to calculate the classification performance 
for each permuted dataset).

An empirical p-value will be computed based on the percentile rank of the 
performance of the  signature on the intact dataset compared to a 
null distribution of
the performance of the signature on all the permuted datasets.
}

\value{
A result list with the following elements:

\itemize{

\item \code{$checkType} is equal to \code{"Permuted"}.

\item \code{$permute} is equal to the passed value of \code{toPermute}.

\item \code{$tests} is the number of tests run (equal to \code{iterations}.)

\item \code{$rank} is the performance rank of the signature on the 
intact dataset compared to its performance in the permuted datasets.

\item \code{$checkPval} is the empirical p-value computed using the performance
scores of the signature on permuted datasets as a null distribution. 
A value of zero indicates that the signature did not perform better on 
any permuted datasets than it does using the intact data.

\item \code{$survivalPval} represents the performance of the primary signature
on the original dataset
if survival data were provided.

\item \code{$survivalPvalsPermuted} is a vector of performance scores (p-values) 
for each permuted dataset, if survival data
were provided.

\item \code{$trainingPvalsPermuted} is a vector of performance scores (p-values) 
for each permuted dataset, if survival data
and separate validation samples were provided.

\item \code{$sigPerformance} is the proportion of validation samples 
correctly classified using the intact dataset if a classifier was used.

\item \code{$modePerformance} is the proportion of validation samples 
correctly classified in the intact dataset using a mode classifier.

\item \code{$performancePermuted} is a vector of classification 
performance scores for each
permuted dataset, each indicating the proportion
of validation samples correctly classified if a classifier was used.
}
}


\author{
Rory Stark
}

\seealso{
\code{\link{sigCheck}}, \code{\link{sigCheckAll}}, 
\code{\link{sigCheckRandom}}, \code{\link{sigCheckKnown}}
}
\examples{
#Disable parallel so Bioconductor build won't hang
library(BiocParallel)
register(SerialParam())

library(breastCancerNKI)
data(nki)
nki <- nki[,!is.na(nki$e.dmfs)]
data(knownSignatures)
ITERATIONS <- 5 # should be at least 20, 1000 for real checks

## survival analysis
check <- sigCheck(nki, classes="e.dmfs", survival="t.dmfs",
                  signature=knownSignatures$cancer$VANTVEER,
                  annotation="HUGO.gene.symbol",
                  validationSamples=150:319)

par(mfrow=c(1,2))
permutedCategories <- sigCheckPermuted(check, toPermute="categories", 
                                       iterations=ITERATIONS)
permutedCategories$checkPval
sigCheckPlot(permutedCategories)
permutedSurvival <- sigCheckPermuted(check, toPermute="survival", 
                                     iterations=ITERATIONS)
permutedSurvival$checkPval
sigCheckPlot(permutedSurvival)
}

\name{nkiResults}
\alias{nkiResults}
\docType{data}
\title{
Precomputed list of results for a call to \code{\link{sigCheckAll}} using the 
\code{breastCancerNKI} dataset.
}
\description{
This object represents the results lists returned by a call to 
\code{\link{sigCheckAll}}. It is used by the vignette accompanying the 
\code{\link{SigCheck}} package as an example result. It was derived by 
running the code in the example below.
}
\usage{data(nkiResults)}

\examples{
\dontrun{
# This is how nkiResults is built
library(breastCancerNKI)
data(nki)
nki = nki[,!is.na(nki$e.dmfs)]
data(knownSignatures)
check <- sigCheck(nki, classes="e.dmfs", survival="t.dmfs",
                  signature=knownSignatures$cancer$VANTVEER,
                  annotation="HUGO.gene.symbol",
                  validationSamples=which(nki$series=="NKI2"))
                  
nkiResults <- sigCheckAll(check,iterations=1000) 
}

# Example usage of nkiResults
data(nkiResults)
sigCheckPlot(nkiResults)
}
\keyword{datasets}

\name{classifyResults}
\alias{classifyResults}
\docType{data}
\title{
Precomputed list of results for a calls to \code{\link{sigCheckRandom}}
and \code{\link{sigCheckKnown}} using the 
\code{breastCancerNKI} dataset.
}
\description{
This object represents the results lists returned by calls to 
\code{\link{sigCheckRandom}} and \code{\link{sigCheckKnown}}. 
It is used by the vignette accompanying the 
\code{\link{SigCheck}} package as an example result. It was derived by 
running the code in the example below.
It loads to object, \code{slassifyRandom} and \code{classifyKnown}.
}
\usage{data(classifyResults)}

\examples{
\dontrun{
# This is how classifyResults is built
library(breastCancerNKI)
data(nki)
nki = nki[,!is.na(nki$e.dmfs)]
data(knownSignatures)
check <- sigCheck(nki, classes="e.dmfs",
                  signature=knownSignatures$cancer$VANTVEER,
                  annotation="HUGO.gene.symbol",
                  validationSamples=101:ncol(nki),
                  scoreMethod="classifier")
                  
classifyRandom <- sigCheckRandom(check, iterations=1000)
classifyKnown <- sigCheckKnown(check)
}

# Example usage of classifyResults
data(classifyResults)
par(mfrow=c(1,2))
sigCheckPlot(classifyRandom, classifier=TRUE)
sigCheckPlot(classifyKnown, classifier=TRUE)

}
\keyword{datasets}

\name{sigCheckPlot}
\alias{sigCheckPlot}
\title{
Plot results of a signature check or set of checks
}
\description{
Plots results of a signature check, as returned by \code{\link{sigCheckRandom}},
\code{\link{sigCheckKnown}}, \code{\link{sigCheckPermuted}}, or 
\code{\link{sigCheckAll}}.
}

\usage{
sigCheckPlot(checkResults, classifier=FALSE, 
             title, nolegend=FALSE, ...)
}

\arguments{
  \item{checkResults}{
The list  returned by \code{\link{sigCheckRandom}}, 
\code{\link{sigCheckKnown}}, \code{\link{sigCheckPermuted}},
or \code{\link{sigCheck}}. 
}
\item{classifier}{
If a classifier was used in the original call to \code{\link{sigCheck}}, setting this
to \code{TRUE} will result in a plot showing how the primary signature compares
based on classification performance (rather than survival).
}
\item{title}{
Title string for plot. If missing, a default plot title will be generated.
}
\item{nolegend}{
IF \code{TRUE}, no legend will be included with the plot(s).
}

  \item{\dots}{
Additional arguments to be passed to the \code{\link{plot}} function.
}
}
\details{
For results based on survival analysis, the background distribution of p-values (in \code{1-log10()} format)
derived from the check (either random signatures, known signatures, or performance
using permuted data) is plotted. Up to two vertical red lines are also plotted: a solid red line representing the performance of the primary signature/data, 
and a dotted red line representing a p-value of 0.05. 
One or both of these may be missing if their performance falls outside the range
of the background distributions.

For results based on classification performance, the x-axis represents the 
range of classification performance scores computed in the check, and the
y-axis representing how many times that score was obtained. In addition, 
vertical lines are plotted representing the classification performance of
the originally specified signature (solid red line) and the performance of 
a classifier that always predicts the mode value of the training samples
(dotted red line).

If the results of \code{\link{sigCheckAll}} is passed in, all four results
plots are generated in a 2x2 grid.
}
\value{
none
}

\note{
Better formance of the signature being checked results in the solid red line 
being to the right of the background distribution.
For survival results, this indicates a lower-value.
For classification results,this indicates superior classification performance.
}
\author{
Rory Stark with Justin Norden
}

\seealso{
\code{\link{sigCheck}}, 
\code{\link{sigCheckAll}},
\code{\link{sigCheckRandom}}, 
\code{\link{sigCheckKnown}}, 
\code{\link{sigCheckPermuted}}
}

\examples{
#Disable parallel so Bioconductor build won't hang
library(BiocParallel)
register(SerialParam())

library(breastCancerNKI)
data(nki)
nki <- nki[,!is.na(nki$e.dmfs)]
data(knownSignatures)
ITERATIONS <- 5 # should be at least 1000 for real checks

## survival analysis with separate training and validation using SVM
check <- sigCheck(nki, classes="e.dmfs", survival="t.dmfs",
                  signature=knownSignatures$cancer$VANTVEER,
                  annotation="HUGO.gene.symbol",
                  validationSamples=250:319,
                  scoreMethod="classifier", threshold=.33) 
                  
results <- sigCheckRandom(check,iterations=ITERATIONS)
par(mfrow=c(1,2))
sigCheckPlot(results)
sigCheckPlot(results, classifier=TRUE)
}

\name{sigCheckAll}
\alias{sigCheckAll}
\title{
Run a default set of checks on a gene signature.
}
\description{
High-level function for package \code{\link{SigCheck}} that runs a default
set of checks against a predictive signature.
}
\usage{
sigCheckAll(check,
            iterations=10, known="cancer", 
            plotResults=TRUE, ...)
}

\arguments{
  \item{check}{
A \code{\link{SigCheckObject}}, as returned by \code{\link{sigCheck}}.
}
  \item{iterations}{
Number of iterations to run to generate background distributions. 
This is how many random signatures the primary signature will be compared to, 
as well as how many of each type of permuted dataset will 
be generated for comparison.
}
  \item{known}{
Specification of a set of known (previously identified) signatures to 
compare to. See \code{\link{sigCheckKnown}} for more details.
}
  \item{plotResults}{
By default, plots of the results will be generated 
unless this is set or \code{FALSE}.
}
  \item{...}{
Extra parameters to pass through \code{\link{sigCheckPlot}}.
}
}

\details{
This high-level function will run four checks, plot the results, and
return a consolidated result set.

First, it calls \code{\link{sigCheckRandom}} to compare the performance 
of \code{interations} randomly selected signatures. 

Next, it calls \code{\link{sigCheckKnown}} to check the 
performance of the signature against a database of signatures previously 
identified to discriminate in other domains.

Finally, two calls are made to \code{\link{sigCheckPermuted}} to check the 
performance of randomly permuted metadata and expression data.
The first call permutes the survival data if they
are available (\code{toPermute="survival"}); 
otherwise it permutes the 
category assignments (\code{toPermute="categories"})
The second call permuted the expression value for each gene (permuting
each row in the \code{\link{ExpressionSet}}, 
equivalent to \code{toPermute="features"}).

If \code{plotResults} is \code{TRUE}, the results are plotted. If a classifier
is involved, a set of four classification results are plotted in
a 2x2 grid, showing how the classification performance of the main
signature compares to that of a mode classifier and to the
distribution of performance values observed for
the random and known signature sets, as well as how it performs using the two 
type of permuted dataset. 
If survival data is available, another 2x2 grid is plotted showing how the 
baseline survival p-value compares to a p-value of 0.05 and to the
distribution of p-values observed for the random and known signatures, as well
as for the permuted data.
}
\value{
A list containing four elements, each containing the result of a check.
\itemize{

\item \code{$checkRandom} is the result list returned by 
\code{\link{sigCheckRandom}}.

\item \code{$checkKnown} is the result list returned by 
\code{\link{sigCheckKnown}}.

The third element of the result list will be one of the following:

\item \code{$checkPermutedSurvival} is the result list returned by 
\code{\link{sigCheckPermuted}}
with \code{toPermute="survival"}.

\item \code{$checkPermutedCategories} is the result list returned by 
\code{\link{sigCheckPermuted}}
with \code{toPermute="categories"}.

The fourth element of the list will be:

\item \code{$checkPermutedFeatures} is the result list returned by 
\code{\link{sigCheckPermuted}}
with \code{toPermute="features"}.

}
}
\references{
Venet, David, Jacques E. Dumont, and Vincent Detours. 
"Most random gene expression signatures are significantly associated 
with breast cancer outcome." PLoS Computational Biology 7.10 (2011): e1002240.
}
\author{
Rory Stark
}

\seealso{
\code{\link{sigCheck}}, \code{\link{sigCheckRandom}}, 
\code{\link{sigCheckPermuted}}, \code{\link{sigCheckKnown}}, 
\code{\link{sigCheckPlot}}
}
\examples{
#Disable parallel so Bioconductor build won't hang
library(BiocParallel)
register(SerialParam())

library(breastCancerNKI)
data(nki)
nki <- nki[,!is.na(nki$e.dmfs)]
data(knownSignatures)
ITERATIONS <- 5 # should be at least 20, 1000 for real checks

## survival analysis
check <- sigCheck(nki, classes="e.dmfs", survival="t.dmfs",
                  signature=knownSignatures$cancer$VANTVEER,
                  annotation="HUGO.gene.symbol",
                  validationSamples=150:319)
                  
results <- sigCheckAll(check,iterations=ITERATIONS, 
                       known=knownSignatures$cancer[1:20])                  


## classification analysis
check <- sigCheck(nki, classes="e.dmfs", 
                  signature=knownSignatures$cancer$VANTVEER,
                  annotation="HUGO.gene.symbol",
                  validationSamples=275:319,
                  scoreMethod="classifier")
                  
results <- sigCheckAll(check,iterations=ITERATIONS, 
                       known=knownSignatures$cancer[1:20])  
}

\name{knownSignatures}
\alias{knownSignatures}
\alias{resultsNKI}
\docType{data}
\title{
Previously identified gene signatures for use in \code{\link{sigCheckKnown}}
}
\description{
Previously identified gene signature sets. These include three signatures sets 
from Venet et. al.
}
\usage{data(knownSignatures)}
\format{
The data object \code{knownSignatures} is a list of sets of gene signatures. 
Each set is a list of gene signatures. 
Each signature is a vector of gene names. 

Gene signature sets include:

\itemize{

\item \code{"cancer"}: 48 gene signatures derived from cancer samples, 
from Venet et. al.

\item \code{"proliferation"}: 5 gene signatures comprising genes associated 
with cell proliferation, including a "super signature", from Venet et. al.

\item \code{"non.cancer"}: 3 gene signatures derived from non-cancer sources, 
from Venet et. al.
}
}

\details{
These data are taken directly from the supplemental material for Venet et. al 
"Most random gene expression signatures are significantly associated with 
breast cancer outcome".
}

\source{
http://www.ploscompbiol.org/article/fetchSingleRepresentation.action?uri=info:doi/10.1371/journal.pcbi.1002240.s001
}

\references{
Venet, David, Jacques E. Dumont, and Vincent Detours.
"Most random gene expression signatures are significantly associated with 
breast cancer outcome." PLoS Computational Biology 7.10 (2011): e1002240.
}

\note{
Other signatures of interest can be downloaded at http://www.broad.mit.edu/gsea/downloads.jsp#msigdb 
and read in using the \code{read.gmt} function in the \code{qusage} package.
}

\examples{
data(knownSignatures)
names(knownSignatures)
names(knownSignatures$cancer)
knownSignatures$cancer$VANTVEER
}

\keyword{datasets}
